#!/usr/bin/env python

import argparse
import datetime
import logging
import os
from glob import glob
import numpy as np
from time import time
from multiprocessing import Queue, Pool
import subprocess
import h5py
import re
import psutil
import pyFAI
import sharedmem

data_to_process = Queue()
proc_data = None
mask_shape = None
config = {}
log = logging.getLogger(__name__)


def integrate(args):
    """
    Worker, which perform integration of images from shared queue.

    Result of integration is stored in sharedmem object.

    :param args: List of arguments [index of worker, number of cores to use]
    """
    try:
        i_worker, n_cores = args
        subprocess.check_output(f"taskset -p -c {n_cores * i_worker}-{n_cores * (i_worker + 1) - 1} {os.getpid()}",
                                shell=True)

        # Dioptas is included here after changed the affinity because then it works faster
        from dioptas.model import MaskModel
        from dioptas.model.loader.LambdaLoader import LambdaImage
        from pyFAI.detectors import Detector, ALL_DETECTORS, NexusDetector
        from pyFAI.geometryRefinement import GeometryRefinement

        mask_model = MaskModel()
        mask = None
        pattern_geometry = None

        lambda_img = None
        while not data_to_process.empty():

            try:
                file_list, img_pos, img_id = data_to_process.get(timeout=5)
            except Exception as e:
                # Excepting is taking place is empty() return True for enpty queue.
                pass

            if lambda_img is None or lambda_img.file_list != file_list:
                lambda_img = LambdaImage(file_list=file_list)
                if lambda_img is None:
                    return

            image = lambda_img.get_image(int(img_pos))

            if mask is None and config['mask_file'] is not None:
                mask_model.set_dimension(image.shape)
                mask_model.load_mask(config['mask_file'])
                mask = mask_model.get_mask()
                mask_shape[...] = image.shape

            if pattern_geometry is None:
                # default params are necessary, otherwise fails...
                detector = Detector(pixel1=79e-6, pixel2=79e-6)
                pattern_geometry = GeometryRefinement(wavelength=0.3344e-10, detector=detector,
                                                      poni1=0, poni2=0)
                pattern_geometry.load(config['cal_file'])
                pattern_geometry.detector = Detector(pixel1=pattern_geometry.pixel1, pixel2=pattern_geometry.pixel2)

            ts = time()
            proc_data[img_id] = np.array(pattern_geometry.integrate1d(image, config['num_points'],
                                                     method=config['int_method'],
                                                     unit='2th_deg',
                                                     azimuth_range=None,
                                                     mask=mask,
                                                     polarization_factor=0.99,
                                                     correctSolidAngle=True,
                                                     filename=None))
            log.info(f"Integrate img {img_id}: {(time() - ts):0.3f}s ")

    except Exception as e:
        log.error("Processing fail: {e}", exc_info=True)
        return False
    return True


def fill_queue(raw_files):
    """
    Read number of images in each file-set (3 files for 3 lambda modules) and fill shared queue
    with image indices and corresponding data file

    :param raw_files: List of raw files to process
    :return: number of images, map of files, map of images
    """

    # Group files by file-sets. Check, that all 3 lambda files are present.
    file_sets = []
    for file in sorted(raw_files, key=lambda x: x[-10:]):

        # avoid duplications
        if len(file_sets) > 0 and file in file_sets[-1]:
            continue

        # all 3 files already in a list
        if (re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>1\g<2>', file) in raw_files and
                re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>2\g<2>', file) in raw_files and
                re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>3\g<2>', file) in raw_files):

            file_sets.append([re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>1\g<2>', file),
                              re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>2\g<2>', file),
                              re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>3\g<2>', file)])
        # Files are not in list but exists
        else:
            files = glob(re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>*\g<2>', file))
            if len(files) == 3:
                file_sets.append(sorted(files))

    img_id = 0
    file_map = [0]
    pos_map = []
    for file_set in file_sets:
        data_path = 'entry/instrument/detector/data'
        lambda_file = h5py.File(file_set[0], "r")
        series_max = lambda_file[data_path].shape[0]
        for img_pos in range(series_max):
            data_to_process.put((file_set, img_pos, img_id))
            img_id += 1
        file_map.append(img_id)
        pos_map += list(zip([len(file_map) - 1] * series_max, range(series_max)))
        lambda_file.close()

    return file_sets, img_id, file_map, pos_map


def get_batches(files):
    """
    Create list of batches and their files

    Each batch is one acquisition.

    :param files: List of filepath templates. Each filepath template should be supported by glob.
    :return: List of batches, list of files for each batch
    """

    file_list = []
    name_list = []
    batch_files = []
    for template in files:
        file_list += glob(template)
    file_list = sorted(list(set(file_list)))

    for file_name in file_list:
        batch_name = re.sub('(.+_m)\d((_part\d+|).nxs)', '\g<1>', file_name)[:-2]
        if batch_name not in name_list:
            name_list.append(batch_name)
            batch_files.append([])
        batch_files[name_list.index(batch_name)].append(file_name)

    return name_list, batch_files


def save_proc_data(filename, files, file_map, pos_map):
    """
    Save diffraction patterns to h5 file

    :param filename: Name of output file
    :param files: List of processed files
    :param file_map: Map of files
    :param pos_map: Map of images in the file
    """
    os.makedirs(os.path.dirname(filename), exist_ok=True)
    with h5py.File(filename, mode="w") as f:
        f.attrs['default'] = 'processed'
        f.attrs['file_time'] = datetime.datetime.now().isoformat()
        f.attrs['pyFAI_version'] = pyFAI.version

        nxentry = f.create_group('processed')
        nxentry.attrs["NX_class"] = 'NXentry'
        nxentry.attrs['default'] = 'result'

        nxdata = nxentry.create_group('result')
        nxdata.attrs["NX_class"] = 'NXdata'
        nxdata.attrs["signal"] = 'data'
        nxdata.attrs["axes"] = ['.', 'binning']

        nxprocess = nxentry.create_group('process')
        nxprocess.attrs["NX_class"] = 'NXprocess'

        nxprocess['cal_file'] = config['cal_file']
        nxprocess['int_method'] = config['int_method']
        nxprocess['int_unit'] = '2th_deg'
        nxprocess['num_points'] = config['num_points']

        if config['mask_file'] is not None:
            nxprocess['mask_file'] = config['mask_file']
            nxprocess['mask_shape'] = mask_shape

        nxdata.create_dataset("data", data=proc_data[:, 1, :])
        tth = nxdata.create_dataset("binning", data=proc_data[0, 0, :])
        tth.attrs["unit"] = 'deg'
        tth.attrs['long_name'] = 'two_theta (degrees)'

        nxprocess.create_dataset("pos_map", data=np.array(pos_map))
        nxprocess.create_dataset("file_map", data=np.array(file_map))
        nxprocess.create_dataset("files", data=np.array(files).astype('S'))


def run():
    """
    Process all given raw data using multiprocessing pool for parallel processing.

    Result of data processing (integration patterns) is stored together with
    metadata in the nexus file.
    """
    global proc_data
    global mask_shape
    n_cores_all = psutil.cpu_count()
    if config['n_proc'] == 0:
        config['n_proc'] = int(n_cores_all * 0.9 / config['n_cores'])

    args = [[i, config['n_cores']] for i in range(config['n_proc'])]
    name_list, batch_files = get_batches(config['data_path'])
    log.info(f"Total number of batches: {len(name_list)}")

    for i_batch, batch_name in enumerate(name_list):

        ts = time()
        log.info(f"Process batch {i_batch}/{len(name_list)}: {batch_name}")

        try:
            file_sets, n_img, file_map, pos_map = fill_queue(batch_files[i_batch])
            log.info(f"List of files: {file_sets}")
        except Exception as e:
            log.error(f"Reading images for batch {batch_name} fails. {e}")
            continue

        if n_img == 0:
            log.error(f"No images in batch: {batch_name}")
            continue

        proc_data = sharedmem.empty((n_img, 2, config['num_points']), dtype='f4')
        mask_shape = sharedmem.empty(2, dtype='i4')
        log.info(f"Found {n_img} images")

        n_current_proc = min(config['n_proc'], max(1, int(n_img/10.)) )
        log.info(f"Process with {n_current_proc} cores")
        int_time = time()
        with Pool(processes=n_current_proc) as pool:
            status = pool.map(integrate, args)
        int_time = time()-int_time

        if all(status):
            local_path = os.path.basename(batch_name)
            if '/raw/' in batch_name:
                local_path = batch_name[batch_name.find('/raw/')+5:]

            try:
                out_file_name = f"{config['out_path']}/{local_path}_v{config['version']:03d}.nxs"
                save_proc_data(out_file_name, batch_files[i_batch], file_map, pos_map)
                log.info(f"Save file: {out_file_name} {n_img}")
            except Exception as e:
                log.error(f"Saving file {out_file_name} fails: {e}")

            log.info(f"Running time: {time() - ts:0.2f}s for {n_img} images")
            log.info(f"Integration time: {int_time:0.2f}s for {n_img} images")
            log.info(f"Time per image: {int_time / n_img * 1000:0.2f}ms ({n_img})")

    global data_to_process
    del data_to_process
    log.info("All done")


def main():
    """
    Create argparser and logging. Run data processing.
    """
    global config
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description='Process raw images')

    parser.add_argument('--cal_file', type=str, help='Path to poni file', required=True)
    parser.add_argument('--mask_file', type=str, help='Path to mask file')
    parser.add_argument('--int_method', type=str, help='Integration method',
                        default='csr')
    parser.add_argument('--num_points', type=int, help='Number of points in diffractogram',
                        default=1500)
    parser.add_argument('--out_path', type=str, help='Output path', required=True)
    parser.add_argument('--data_path', type=str, nargs='*',
                        help='Path to raw folder of imput data', required=True)

    parser.add_argument('--n_proc', type=int, help='Number of processes. 0 - 0.9 of all cores',
                        default=0)
    parser.add_argument('--n_cores', type=int, help='Number of cores per process',
                        default=1)
    parser.add_argument('--version', type=int, help='Version of the reprocessing config',
                        default=-1)
    parser.add_argument('--log-file', type=str)
    parser.add_argument("--log_level", default="INFO", choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
                        help="Set log level for the application")

    config = vars(parser.parse_args())

    fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(filename=config["log_file"],
                        level=getattr(logging, config['log_level']),
                        format=fmt)
    run()


if __name__ == "__main__":
    main()



# SPDX-License-Identifier: MIT
import os

import pytest

from ...model.util import jcpds as jcpds_class

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')
jcpds_path = os.path.join(data_path, 'jcpds')


@pytest.fixture
def jcpds():
    return jcpds_class()


def test_sorting_of_reflections(jcpds):
    jcpds.add_reflection(1, 0, 0, 100, 4.0)
    jcpds.add_reflection(1, 2, 0, 90, 2.0)
    jcpds.add_reflection(2, 2, 0, 23, 3.0)
    jcpds.add_reflection(5, 2, 1, 50, 6.0)
    jcpds.add_reflection(3, 2, 2, 10, 41.0)
    jcpds.add_reflection(4, 3, 0, 30, 1.0)
    jcpds.add_reflection(2, 2, 5, 2, 0.3)

    jcpds.sort_reflections_by_h()
    assert jcpds.reflections[0].d0 == 4.0
    assert jcpds.reflections[6].d0 == 6.0

    jcpds.sort_reflections_by_k()
    assert jcpds.reflections[0].d0 == 4.0
    assert jcpds.reflections[6].d0 == 1.0

    jcpds.sort_reflections_by_l()
    assert jcpds.reflections[0].d0 == 4.0
    assert jcpds.reflections[6].d0 == 0.3

    jcpds.sort_reflections_by_intensity()
    assert jcpds.reflections[0].d0 == 0.3
    assert jcpds.reflections[6].d0 == 4.0

    jcpds.sort_reflections_by_d()
    assert jcpds.reflections[0].intensity == 2
    assert jcpds.reflections[6].intensity == 10


def test_modified_flag(jcpds):
    assert not jcpds.params['modified']
    jcpds.params['a0'] = 3
    assert jcpds.params['modified']
    assert jcpds.params['a0'] == 3
    jcpds.modified = False

    jcpds.load_file(os.path.join(jcpds_path, 'au_Anderson.jcpds'))
    assert not jcpds.params['modified']
    jcpds.params['k0'] = 200
    assert jcpds.params['modified']
    assert os.path.join(jcpds_path, 'au_Anderson.jcpds*') == jcpds.filename
    assert 'au_Anderson*' == jcpds.name


def get_reflection_d_spacing(reflections, h, k, l):
    for reflection in reflections:
        if reflection.h == h and reflection.k == k and reflection.l == l:
            return reflection.d0


def test_consistency_d_spacing_calculation(jcpds):
    # loading a monoclinic jcpds and check if different signs will change the d spacing
    jcpds.load_file(os.path.join(jcpds_path, 'FeGeO3_cpx.jcpds'))

    d1_mon = get_reflection_d_spacing(jcpds.reflections, 2, 2, 1)
    d2_mon = get_reflection_d_spacing(jcpds.reflections, -2, 2, 1)

    jcpds.params['symmetry'] = 'TRICLINIC'
    jcpds.compute_d0()

    d1_tri = get_reflection_d_spacing(jcpds.reflections, 2, 2, 1)
    d2_tri = get_reflection_d_spacing(jcpds.reflections, -2, 2, 1)

    assert d1_mon == pytest.approx(d1_tri)
    assert d2_mon == pytest.approx(d2_tri)


def test_using_negative_pressures(jcpds):
    jcpds.load_file(os.path.join(jcpds_path, 'au_Anderson.jcpds'))
    jcpds.pressure = -1.

    jcpds.compute_d(-1, 298)
    assert jcpds.params['v'] > jcpds.params['v0']


def test_using_negative_pressures_with_zero_bulk_modulus(jcpds):
    jcpds.load_file(os.path.join(jcpds_path, 're_K0.jcpds'))
    jcpds.pressure = -1.

    jcpds.compute_d(-1, 298)
    assert jcpds.params['v'] == jcpds.params['v0']



# SPDX-License-Identifier: MIT

import numpy as np

s2pi = np.sqrt(2 * np.pi)
def gaussian(x, amplitude=1.0, center=0.0, sigma=1.0):
    """1 dimensional gaussian:
    gaussian(x, amplitude, center, sigma)
    """
    return (amplitude / (s2pi * sigma)) * np.exp(-(1.0 * x - center) ** 2 / (2 * sigma ** 2))




# SPDX-License-Identifier: MIT

from ..utility import QtTest, click_button, unittest_data_path, click_checkbox, delete_if_exists
import os
import gc
import numpy as np
from mock import MagicMock

from qtpy import QtWidgets, QtCore, QtGui
from qtpy.QtTest import QTest

from ...model.DioptasModel import DioptasModel
from ...controller.MaskController import MaskController
from ...widgets.MaskWidget import MaskWidget


class MaskControllerTest(QtTest):
    def setUp(self):

        self.model = DioptasModel()
        self.model.working_directories = {'mask': unittest_data_path}

        self.mask_widget = MaskWidget()
        self.mask_controller = MaskController(self.mask_widget, self.model)

    def tearDown(self):
        delete_if_exists(os.path.join(unittest_data_path, 'dummy.mask'))
        del self.model
        self.mask_widget.close()
        del self.mask_widget
        del self.mask_controller
        gc.collect()

    def get_file_size(self, filename):
        stat_info = os.stat(filename)
        return stat_info.st_size

    def test_loading_and_saving_mask_files(self):
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(return_value=(
            os.path.join(unittest_data_path, 'test.mask'),
            MaskController.DEFAULT_MASK_FILTER,
        ))
        click_button(self.mask_widget.load_mask_btn)
        self.model.mask_model.mask_below_threshold(self.model.img_data, 1)

        dialog_results = (
            ('.mask', MaskController.DEFAULT_MASK_FILTER),
            ('.npy', f"{MaskController.FLIPUD_MASK_FILTER_PREFIX} (*.npy)"),
            ('.edf', f"{MaskController.FLIPUD_MASK_FILTER_PREFIX} (*.edf)")

        )
        for extension, selected_filter in dialog_results:
            with self.subTest(extension=extension, selected_filter=selected_filter):
                filename = os.path.join(unittest_data_path, f'dummy{extension}')
                QtWidgets.QFileDialog.getSaveFileName = MagicMock(return_value=(filename, selected_filter))
                click_button(self.mask_widget.save_mask_btn)
                self.assertTrue(os.path.exists(filename))
                delete_if_exists(filename)

    def test_grow_and_shrinking(self):
        self.model.mask_model.mask_ellipse(100, 100, 20, 20)
        previous_mask = np.copy(self.model.mask_model._mask_data)

        QTest.mouseClick(self.mask_widget.grow_btn, QtCore.Qt.LeftButton)
        self.assertFalse(np.array_equal(previous_mask, self.model.mask_model._mask_data))

        QTest.mouseClick(self.mask_widget.shrink_btn, QtCore.Qt.LeftButton)
        self.assertTrue(np.array_equal(previous_mask, self.model.mask_model._mask_data))

    def test_mask_and_unmask(self):
        # test that changing mask mode modifies the model and the color in img_widget
        self.mask_widget.mask_rb.click()
        self.assertEqual(self.model.mask_model.mode, True)
        self.assertEqual(self.mask_widget.img_widget.mask_preview_fill_color, QtGui.QColor(255, 0, 0, 150))
        self.mask_widget.unmask_rb.click()
        self.assertEqual(self.model.mask_model.mode, False)
        self.assertEqual(self.mask_widget.img_widget.mask_preview_fill_color, QtGui.QColor(0, 255, 0, 150))

        # test that masking and unmasking the same area results in the same mask
        previous_mask = np.copy(self.model.mask_model._mask_data)
        self.mask_widget.mask_rb.click()
        self.model.mask_model.mask_ellipse(100, 100, 20, 20)
        self.assertFalse(np.array_equal(previous_mask, self.model.mask_model._mask_data))
        self.mask_widget.unmask_rb.click()
        self.model.mask_model.mask_ellipse(100, 100, 20, 20)
        self.assertTrue(np.array_equal(previous_mask, self.model.mask_model._mask_data))

    def test_select_configuration_updating_mask(self):
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(return_value=(
            os.path.join(unittest_data_path, 'test.mask'),
            MaskController.DEFAULT_MASK_FILTER,
        ))
        click_button(self.mask_widget.load_mask_btn)
        first_mask = self.model.mask_model.get_img()
        self.model.add_configuration()
        self.model.mask_model.mask_below_threshold(self.model.img_data, 1)
        second_mask = self.model.mask_model.get_img()

        self.model.select_configuration(0)
        self.assertEqual(np.sum(self.mask_widget.img_widget.mask_img_item.image-first_mask), 0)
        self.model.select_configuration(1)
        self.assertEqual(np.sum(self.mask_widget.img_widget.mask_img_item.image-second_mask), 0)

    def test_select_configuration_updating_mask_transparency(self):
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(return_value=(
            os.path.join(unittest_data_path, 'test.mask'),
            MaskController.DEFAULT_MASK_FILTER,
        ))
        click_button(self.mask_widget.load_mask_btn)
        self.model.add_configuration()
        self.model.mask_model.mask_below_threshold(self.model.img_data, 1)
        click_checkbox(self.mask_widget.transparent_rb)
        self.assertTrue(self.mask_widget.transparent_rb.isChecked())

        transparent_color = self.mask_widget.img_widget.create_color_map([255, 0, 0, 100])
        filled_color = self.mask_widget.img_widget.create_color_map([255, 0, 0, 255])

        self.assertTrue(self.model.transparent_mask)
        self.model.select_configuration(0)
        self.assertTrue(self.mask_widget.fill_rb.isChecked())
        self.assertTrue(np.array_equal(self.mask_widget.img_widget.mask_img_item.lut, filled_color))
        self.model.select_configuration(1)
        self.assertTrue(self.mask_widget.transparent_rb.isChecked())
        self.assertTrue(np.array_equal(self.mask_widget.img_widget.mask_img_item.lut, transparent_color))

    def test_apply_cosmic_removal(self):
        click_button(self.mask_widget.cosmic_btn)



# SPDX-License-Identifier: MIT

import pytest
import os
import numpy as np
from mock import MagicMock

from qtpy import QtWidgets
from xypattern import Pattern

from dioptas.tests.utility import enter_value_into_text_field, click_button

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, os.pardir, "data")


@pytest.fixture
def mock_integration(dioptas_model):
    pattern = Pattern(x=np.linspace(0, 35, 2500), y=np.random.random(2500) * 1000)
    dioptas_model.calibration_model.integrate_1d = MagicMock(
        return_value=(pattern.x, pattern.y)
    )
    dioptas_model.calibration_model.load(os.path.join(data_path, "CeO2_Pilatus1M.poni"))
    dioptas_model.img_model.load(os.path.join(data_path, "CeO2_Pilatus1M.tif"))


def test_1D_integration_with_azimuth_limits(
    integration_controller, integration_widget, dioptas_model, mock_integration
):
    # Edith wants to perform 1D integration within a certain range of azimuthal angles. She sees there is an option
    # in the X tab and deselects the Full Range button, enabling the text edits and then manually inputs -100, 80
    assert not integration_widget.oned_azimuth_min_txt.isEnabled()
    assert not integration_widget.oned_azimuth_max_txt.isEnabled()
    integration_widget.oned_full_range_btn.setChecked(False)
    assert integration_widget.oned_azimuth_min_txt.isEnabled()
    assert integration_widget.oned_azimuth_max_txt.isEnabled()

    enter_value_into_text_field(integration_widget.oned_azimuth_min_txt, -100)
    enter_value_into_text_field(integration_widget.oned_azimuth_max_txt, 80)

    dioptas_model.calibration_model.integrate_1d.assert_called_with(
        mask=None,
        num_points=None,
        unit="2th_deg",
        azi_range=(-100, 80),
        trim_zeros=True,
    )


def test_changing_number_of_integration_bins(
    integration_controller, integration_widget, dioptas_model, mock_integration
):
    # Edith wants to change the number of integration bins in order to see the effect of binning onto her line
    # shape. She sees that there is an option in the X tab and deselects automatic and sees that the spinbox
    # becomes editable.
    assert not integration_widget.bin_count_txt.isEnabled()
    integration_widget.automatic_binning_cb.setChecked(False)
    assert integration_widget.bin_count_txt.isEnabled()

    # she sees that the current value and wants to double it and notices that the pattern looks a little bit
    # smoother
    previous_number_of_points = len(dioptas_model.pattern.x)
    enter_value_into_text_field(
        integration_widget.bin_count_txt, 2 * previous_number_of_points
    )

    dioptas_model.calibration_model.integrate_1d.assert_called_with(
        num_points=2 * previous_number_of_points,
        azi_range=None,
        mask=None,
        unit="2th_deg",
        trim_zeros=True,
    )

    # then she decides that having an automatic estimation may probably be better and changes back to automatic.
    # immediately the number is restored and the image looks like when she started
    integration_widget.automatic_binning_cb.setChecked(True)
    dioptas_model.calibration_model.integrate_1d.assert_called_with(
        num_points=None, azi_range=None, mask=None, unit="2th_deg", trim_zeros=True
    )


def test_changing_supersampling_amount_integrating_to_cake_with_mask(
    integration_controller, integration_widget, dioptas_model, mock_integration
):
    # Edith opens the program, calibrates everything and looks in to the options menu. She sees that there is a
    # miraculous parameter called supersampling. It is currently set to 1 which seems to be normal
    assert integration_widget.supersampling_sb.value() == 1

    # then she sets it to two and she sees that the number of pattern bin changes and that the pattern looks
    # smoother

    # values before:
    px1 = dioptas_model.calibration_model.pattern_geometry.pixel1
    px2 = dioptas_model.calibration_model.pattern_geometry.pixel2

    integration_widget.supersampling_sb.setValue(2)
    assert dioptas_model.calibration_model.pattern_geometry.pixel1 == 0.5 * px1
    assert dioptas_model.calibration_model.pattern_geometry.pixel2 == 0.5 * px2
    assert dioptas_model.calibration_model.cake_geometry.pixel1 == 0.5 * px1
    assert dioptas_model.calibration_model.cake_geometry.pixel2 == 0.5 * px2


def test_saving_image(
    integration_controller, integration_widget, dioptas_model, mock_integration, tmpdir
):
    # the widget has to be shown to be able to save the image:
    integration_widget.show()
    # Tests if the image save procedures are working for the different possible file endings
    QtWidgets.QFileDialog.getSaveFileName = MagicMock(
        return_value=os.path.join(tmpdir, "Test_img.png")
    )
    click_button(integration_widget.qa_save_img_btn)
    QtWidgets.QFileDialog.getSaveFileName = MagicMock(
        return_value=os.path.join(tmpdir, "Test_img.tiff")
    )
    click_button(integration_widget.qa_save_img_btn)

    assert os.path.exists(os.path.join(tmpdir, "Test_img.png"))
    assert os.path.exists(os.path.join(tmpdir, "Test_img.tiff"))


@pytest.mark.parametrize("integration_variable", ["2th", "q", "d"])
def test_saving_pattern(
    integration_controller,
    integration_widget,
    mock_integration,
    tmpdir,
    integration_variable,
):
    # the widget has to be shown to be able to save the image:
    integration_widget.show()
    pattern_controller = integration_controller.pattern_controller

    if integration_variable == "d":
        click_button(pattern_controller.widget.pattern_d_btn)
    elif integration_variable == "q":
        click_button(pattern_controller.widget.pattern_q_btn)

    def save_pattern(filename):
        QtWidgets.QFileDialog.getSaveFileName = MagicMock(return_value=filename)
        click_button(integration_widget.qa_save_pattern_btn)

    for file_format in ("xy", "chi", "dat", "png"):  # SVG does not work currently
        filename = os.path.join(tmpdir, "test_pattern." + file_format)
        save_pattern(filename)
        assert os.path.exists(filename)
        assert os.stat(filename).st_size > 1


def test_undocking_and_docking_img_frame(
    integration_controller, integration_widget, mock_integration
):
    # we click the dock btn twice to undock and dock the image widget
    click_button(integration_widget.img_dock_btn)
    click_button(integration_widget.img_dock_btn)


def test_loading_multiple_images_and_batch_integrate_them(
    integration_widget, integration_controller, mock_integration, tmp_path
):
    QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
        return_value=[
            os.path.join(data_path, "image_001.tif"),
            os.path.join(data_path, "image_002.tif"),
        ]
    )
    QtWidgets.QFileDialog.getExistingDirectory = MagicMock(return_value=str(tmp_path))
    click_button(integration_widget.load_img_btn)

    assert os.path.exists(os.path.join(tmp_path, "image_001.xy"))
    assert os.path.exists(os.path.join(tmp_path, "image_002.xy"))


def test_loading_multiple_images_and_batch_save_them(
    integration_widget, integration_controller, mock_integration, tmp_path
):
    QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
        return_value=[
            os.path.join(data_path, "image_001.tif"),
            os.path.join(data_path, "image_002.tif"),
        ]
    )
    QtWidgets.QFileDialog.getExistingDirectory = MagicMock(return_value=str(tmp_path))
    integration_widget.img_batch_mode_image_save_rb.setChecked(True)
    click_button(integration_widget.load_img_btn)

    assert os.path.exists(os.path.join(tmp_path, "batch_image_001.tif"))
    assert os.path.exists(os.path.join(tmp_path, "batch_image_002.tif"))




# SPDX-License-Identifier: MIT

import fabio


class FabioLoader:
    def __init__(self, filename):
        """
        Loads an Hdf5 image produced by ESRF
        :param filename: path to the hdf5 file to be loaded
        """

        self.fabio_image = fabio.open(filename)
        self.series_max = self.fabio_image.nframes

    def get_image(self, ind=0):
        return self.fabio_image.get_frame(ind).data[::-1]



# SPDX-License-Identifier: MIT

import unittest
from ..utility import QtTest
from qtpy.QtTest import QTest
from qtpy import QtCore, QtGui
import os
import gc
from qtpy import QtWidgets

from mock import MagicMock
import numpy as np

from ..utility import click_button, enter_value_into_text_field, delete_if_exists
from ...controller.integration import JcpdsEditorController
from ...model.DioptasModel import DioptasModel
from ...widgets.integration import IntegrationWidget

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')
jcpds_path = os.path.join(data_path, 'jcpds')


class JcpdsEditorControllerTest(QtTest):
    # SETUP
    #######################
    def setUp(self) -> None:
        self.model = DioptasModel()
        self.model.calibration_model.is_calibrated = True
        self.model.calibration_model.pattern_geometry.wavelength = 0.31E-10
        self.model.calibration_model.integrate_1d = MagicMock(return_value=(self.model.calibration_model.tth,
                                                                            self.model.calibration_model.int))

        self.phase_model = self.model.phase_model

        self.widget = IntegrationWidget()

        self.controller = JcpdsEditorController(self.widget, self.model)
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))

        self.jcpds_widget = self.controller.jcpds_widget
        self.phase_widget = self.widget.phase_widget

        self.load_phases()
        self.controller.active = True
        self.controller.show_view()

        self.setup_selected_row(5)
        click_button(self.phase_widget.edit_btn)

    def tearDown(self) -> None:
        del self.controller
        del self.widget
        self.model.delete_configurations()
        del self.model
        delete_if_exists(os.path.join(jcpds_path, 'dummy.jcpds'))
        delete_if_exists(os.path.join(data_path, 'reflection_table.txt'))
        gc.collect()

    # Utility Functions
    #######################
    def load_phase(self, filename):
        self.model.phase_model.add_jcpds(os.path.join(jcpds_path, filename))

    def load_phases(self):
        self.load_phase('ar.jcpds')
        self.load_phase('ag.jcpds')
        self.load_phase('au_Anderson.jcpds')
        self.load_phase('mo.jcpds')
        self.load_phase('pt.jcpds')
        self.load_phase('re.jcpds')

    def setup_selected_row(self, ind):
        self.phase_widget.get_selected_phase_row = MagicMock(return_value=ind)

    def send_phase_tw_select_signal(self, ind):
        self.phase_widget.phase_tw.currentCellChanged.emit(ind, 0, 0, 0)

    def set_symmetry(self, symmetry):
        self.jcpds_widget.symmetry_cb.setCurrentIndex(self.jcpds_widget.symmetries.index(symmetry))
        QtWidgets.QApplication.processEvents()

    # Tests
    #######################
    def test_edit_button_shows_correct_phase(self):
        self.assertEqual(self.jcpds_widget.filename_txt.text(), self.phase_model.phases[5].filename)

    def test_selection_changed_shows_correct_phase(self):
        self.send_phase_tw_select_signal(3)
        self.assertEqual(self.jcpds_widget.filename_txt.text(), self.phase_model.phases[3].filename)

    def test_updating_the_gui_after_external_change(self):
        previous_a = self.jcpds_widget.lattice_eos_a_txt.text()
        self.phase_model.set_pressure(5, 20)
        self.assertNotEqual(previous_a, self.jcpds_widget.lattice_eos_a_txt.text())

    def test_updating_volume_after_changing_a(self):
        previous_volume = self.jcpds_widget.lattice_volume_txt.text()
        self.jcpds_widget.lattice_a_sb.setValue(3)
        self.assertNotEqual(previous_volume, self.jcpds_widget.lattice_volume_txt.text())

    def test_updating_volume_after_changing_c(self):
        previous_volume = self.jcpds_widget.lattice_volume_txt.text()
        self.jcpds_widget.lattice_c_sb.setValue(3)
        self.assertNotEqual(previous_volume, self.jcpds_widget.lattice_volume_txt.text())

    def test_updating_volume_after_changing_ca_ratio(self):
        previous_volume = self.jcpds_widget.lattice_volume_txt.text()
        self.jcpds_widget.lattice_c_sb.setValue(3)
        self.assertNotEqual(previous_volume, self.jcpds_widget.lattice_volume_txt.text())

    def test_updating_k0_parameter(self):
        self.phase_model.set_pressure(5, 30)
        previous_volume = self.jcpds_widget.lattice_eos_volume_txt.text()
        enter_value_into_text_field(self.jcpds_widget.eos_K_txt, 300)
        self.assertNotEqual(previous_volume, self.jcpds_widget.lattice_eos_volume_txt.text())

    def test_updating_kp_parameter(self):
        self.phase_model.set_pressure(5, 30)
        previous_volume = self.jcpds_widget.lattice_eos_volume_txt.text()
        enter_value_into_text_field(self.jcpds_widget.eos_Kp_txt, 5)
        self.assertNotEqual(previous_volume, self.jcpds_widget.lattice_eos_volume_txt.text())

    def test_adding_a_reflection(self):
        num_phase_reflections = len(self.phase_model.phases[5].reflections)
        num_table_reflections = self.jcpds_widget.reflection_table_model.rowCount()
        self.assertEqual(num_phase_reflections, num_table_reflections)

        click_button(self.jcpds_widget.reflections_add_btn)

        self.assertEqual(len(self.phase_model.phases[5].reflections),
                         num_phase_reflections + 1)
        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(),
                         num_table_reflections + 1)

    def test_adding_two_reflections(self):
        num_phase_reflections = len(self.phase_model.phases[5].reflections)
        num_table_reflections = self.jcpds_widget.reflection_table_model.rowCount()
        self.assertEqual(num_phase_reflections, num_table_reflections)

        click_button(self.jcpds_widget.reflections_add_btn)
        click_button(self.jcpds_widget.reflections_add_btn)

        self.assertEqual(len(self.phase_model.phases[5].reflections),
                         num_phase_reflections + 2)
        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(),
                         num_table_reflections + 2)

    def test_removing_one_reflection(self):
        num_phase_reflections = len(self.phase_model.phases[5].reflections)
        num_table_reflections = self.jcpds_widget.reflection_table_model.rowCount()
        self.assertEqual(num_phase_reflections, num_table_reflections)

        self.jcpds_widget.get_selected_reflections = MagicMock(return_value=[3])

        click_button(self.jcpds_widget.reflections_delete_btn)

        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(),
                         num_table_reflections - 1)
        self.assertEqual(len(self.phase_model.phases[5].reflections),
                         num_phase_reflections - 1)

    def test_removing_multiple_reflections(self):
        num_phase_reflections = len(self.phase_model.phases[5].reflections)
        num_table_reflections = self.jcpds_widget.reflection_table_model.rowCount()
        self.assertEqual(num_phase_reflections, num_table_reflections)

        self.jcpds_widget.reflection_table_view.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        for ind in [3, 1, 5, 11]:
            self.jcpds_widget.reflection_table_view.selectRow(ind)

        click_button(self.jcpds_widget.reflections_delete_btn)

        self.assertEqual(len(self.phase_model.phases[5].reflections),
                         num_phase_reflections - 4)
        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(),
                         num_table_reflections - 4)

    def test_clear_reflections(self):
        self.assertGreater(self.jcpds_widget.reflection_table_model.rowCount(), 0)
        click_button(self.jcpds_widget.reflections_clear_btn)
        self.assertEqual(len(self.phase_model.phases[5].reflections), 0)
        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(), 0)


    @unittest.skip('Does currently not work with pytest running a full suite - needs to be added later again')
    def test_edit_reflection(self):
        col = 0
        row = 1

        previous_d0 = self.phase_model.phases[5].reflections[1].d

        # get x,y position for the cell
        x_pos = self.jcpds_widget.reflection_table_view.columnViewportPosition(col) + 3
        y_pos = self.jcpds_widget.reflection_table_view.rowViewportPosition(row) + 10

        # click then doubleclick  the cell
        self.assertTrue(self.jcpds_widget.reflection_table_view.isVisible())
        viewport = self.jcpds_widget.reflection_table_view.viewport()
        QTest.mouseClick(viewport, QtCore.Qt.LeftButton, pos=QtCore.QPoint(x_pos, y_pos))
        QTest.mouseDClick(viewport, QtCore.Qt.LeftButton, pos=QtCore.QPoint(x_pos, y_pos))

        # type in the new number
        QTest.keyClicks(viewport.focusWidget(), "3")
        QTest.keyPress(viewport.focusWidget(), QtCore.Qt.Key_Enter)
        QtWidgets.QApplication.processEvents()

        self.assertEqual(self.phase_model.phases[5].reflections[1].h, 3)
        print(self.phase_model.phases[5].reflections[1].d)
        self.assertNotEqual(self.phase_model.phases[5].reflections[1].d,
                            previous_d0)

    def test_reload_phase(self):
        num_phase_reflections = len(self.phase_model.phases[5].reflections)
        self.phase_model.delete_reflection(5, 0)
        self.phase_model.delete_reflection(5, 0)
        self.assertEqual(len(self.phase_model.reflections[5]), num_phase_reflections - 2)

        previous_a = self.jcpds_widget.lattice_a_sb.value()
        self.jcpds_widget.lattice_a_sb.setValue(3)

        click_button(self.jcpds_widget.reload_file_btn)

        self.assertEqual(self.jcpds_widget.reflection_table_model.rowCount(), num_phase_reflections)
        self.assertEqual(self.jcpds_widget.lattice_a_sb.value(), previous_a)

    def test_save_as(self):
        filename = os.path.join(jcpds_path, 'dummy.jcpds')
        QtWidgets.QFileDialog.getSaveFileName = MagicMock(return_value=filename)
        click_button(self.jcpds_widget.save_as_btn)
        self.assertEqual(self.jcpds_widget.filename_txt.text(), filename)

    def test_changing_symmetry(self):
        print(self.controller.jcpds_phase.params['symmetry'])
        self.assertTrue(self.jcpds_widget.lattice_a_sb.isEnabled())
        self.assertFalse(self.jcpds_widget.lattice_b_sb.isEnabled())
        self.assertTrue(self.jcpds_widget.lattice_c_sb.isEnabled())

        self.set_symmetry('orthorhombic')
        self.assertEqual(self.controller.jcpds_phase.params['symmetry'], 'ORTHORHOMBIC')
        self.assertTrue(self.jcpds_widget.lattice_a_sb.isEnabled())
        self.assertTrue(self.jcpds_widget.lattice_b_sb.isEnabled())
        self.assertTrue(self.jcpds_widget.lattice_c_sb.isEnabled())

    def test_select_row_in_phase_widget(self):
        self.controller.phase_widget.phase_tw.setRowCount(6)
        self.controller.phase_widget.phase_tw.selectRow(2)
        QtWidgets.QApplication.processEvents()
        self.assertEqual(self.controller.phase_ind, 2)

    def test_editing_the_comments(self):
        enter_value_into_text_field(self.jcpds_widget.comments_txt,
                                    'HAHA this is a phase you will never see in your pattern')
        self.assertEqual(self.controller.jcpds_phase.params['comments'][0],
                         'HAHA this is a phase you will never see in your pattern')

    def test_export_reflection_table(self):
        path = os.path.join(data_path, 'reflection_table.txt')
        self.controller.export_table_data(path)
        with open(path, 'r') as f:
            self.assertEqual(len(f.readlines()), 21)

    def test_copy_complete_reflection_table(self):
        self.controller.reflection_table_key_pressed(QtGui.QKeySequence.SelectAll)
        self.controller.reflection_table_key_pressed(QtGui.QKeySequence.Copy)

        self.assertEqual(QtWidgets.QApplication.clipboard().text().split('\n')[3],
                         '1	0	1	100.00	2.1065	2.1065	8.4396	8.4396	2.9828	2.9828')

        self.jcpds_widget.reflection_table_model.wavelength = None
        self.controller.reflection_table_key_pressed(QtGui.QKeySequence.Copy)
        self.assertEqual(QtWidgets.QApplication.clipboard().text().split('\n')[3],
                         '1	0	1	100.00	2.1065	2.1065')




# SPDX-License-Identifier: MIT

import os
import gc
import tempfile
import uuid
from collections import OrderedDict

import numpy as np

from mock import MagicMock, patch

from qtpy import QtWidgets

import fabio

from ...model.CalibrationModel import CalibrationModel
from ...model.util.HelperModule import rotate_matrix_m90, rotate_matrix_p90
from ...controller.MainController import MainController
from ..utility import (
    QtTest,
    click_button,
    delete_if_exists,
    enter_value_into_text_field,
)

from ... import calibrants_path

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, "../data")
jcpds_path = os.path.join(data_path, "jcpds")

working_directories = {
    "image": data_path,
    "calibration": data_path,
    "phase": os.path.join(data_path, "jcpds"),
    "overlay": data_path,
    "mask": data_path,
    "pattern": data_path,
    "project": data_path,
}

integration_unit = "q_A^-1"
use_mask = True
transparent_mask = True
roi = (100, 120, 300, 400)
auto_save_integrated_patterns = True
integrated_patterns_file_formats = [".xy", ".chi"]
img_autoprocess = True
detector_thickness = 30
absorption_length = 175
test_image_file_name = os.path.join(data_path, "CeO2_Pilatus1M.tif")
test_image_file_name_2 = os.path.join(data_path, "a_CeO2_Pilatus1M.tif")
test_calibration_file = os.path.join(data_path, "CeO2_Pilatus1M.poni")
test_calibration_file_2 = os.path.join(data_path, "a_CeO2_Pilatus1M.poni")
poly_order = 55
x_min = 1.0
x_max = 8.0
pyfai_params = OrderedDict(
    {
        "detector": "Detector",
        "pixel1": 7.9e-05,
        "pixel2": 7.9e-05,
        "dist": 0.196711580484,
        "poni1": 0.0813975852141,
        "poni2": 0.0820662115429,
        "rot1": 0.00615439716514,
        "rot2": -0.00156720465515,
        "rot3": 1.68707221612e-06,
        "wavelength": 3.1e-11,
        "polarization_factor": 0.99,
    }
)
pressure = 12.0


class ProjectSaveLoadTest(QtTest):
    def setUp(self):
        self.controller = MainController(use_settings=False)
        self.model = self.controller.model
        self.widget = self.controller.widget
        self.config_widget = self.controller.widget.configuration_widget
        self.config_controller = self.controller.configuration_controller
        self.check_calibration = True
        self.maxDiff = None
        self.config_file_path = os.path.join(
            tempfile.gettempdir(),
            f"dioptas_test_save_load_{os.getpid()}_{uuid.uuid4().hex}.hdf5",
        )

    def tearDown(self):
        delete_if_exists(os.path.join(data_path, "CeO2_Pilatus1M.chi"))
        delete_if_exists(os.path.join(data_path, "CeO2_Pilatus1M.xy"))
        delete_if_exists(self.config_file_path)
        self.resetState()

    def resetState(self):
        if self.model.calibration_model.cake_geometry is not None:
            self.model.calibration_model.cake_geometry.reset()
        self.model.calibration_model.pattern_geometry.reset()
        self.model.disconnect_models()

        # Close widgets before deletion
        self.config_widget.close()
        self.widget.integration_widget.close()
        self.widget.mask_widget.close()
        self.widget.calibration_widget.close()
        self.widget.close()

        self.config_widget.deleteLater()
        self.widget.integration_widget.deleteLater()
        self.widget.integration_widget.integration_control_widget.deleteLater()
        self.widget.integration_widget.integration_image_widget.deleteLater()
        self.widget.integration_widget.integration_pattern_widget.deleteLater()
        self.widget.integration_widget.integration_status_widget.deleteLater()
        self.widget.mask_widget.deleteLater()
        self.widget.calibration_widget.deleteLater()
        self.widget.deleteLater()

        # Process events to ensure widgets are cleaned up
        QtWidgets.QApplication.processEvents()

        del self.config_widget
        del self.widget.integration_widget.integration_control_widget
        del self.widget.integration_widget.integration_image_widget
        del self.widget.integration_widget.integration_pattern_widget
        del self.widget.integration_widget.integration_status_widget
        del self.widget.integration_widget
        del self.widget.mask_widget
        del self.widget.calibration_widget
        del self.widget

        del self.config_controller
        del self.controller
        del self.model
        gc.collect()

        # Process events again after deletion
        QtWidgets.QApplication.processEvents()

    def load_image(self, file_name):
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(return_value=[file_name])
        click_button(
            self.controller.integration_controller.widget.load_img_btn
        )  # load file

        self.model.current_configuration.calibration_model.set_pyFAI(pyfai_params)
        self.model.working_directories = working_directories
        self.model.current_configuration.integration_unit = integration_unit
        self.raw_img_data = self.model.current_configuration.img_model.raw_img_data

    def save_and_load_configuration(
        self, prepare_function, intermediate_function=None, mock_1d_integration=True
    ):
        saved_config_file_path = self.config_file_path
        if mock_1d_integration:
            with patch.object(
                CalibrationModel,
                "integrate_1d",
                return_value=(np.linspace(0, 20, 1001), np.ones((1001,))),
            ):
                self.load_image(test_image_file_name)
                if prepare_function is not None:
                    prepare_function()
                self.save_configuration()
                self.model.reset()
                self.model.working_directories = {
                    "calibration": "",
                    "mask": "",
                    "image": os.path.expanduser("~"),
                    "pattern": "",
                    "overlay": "",
                    "phase": "",
                }
                self.resetState()
                self.setUp()
                self.config_file_path = saved_config_file_path
                if intermediate_function:
                    intermediate_function()

                self.load_configuration()
        else:
            self.load_image(test_image_file_name)
            if prepare_function is not None:
                prepare_function()
            self.save_configuration()
            self.model.reset()
            self.model.working_directories = {
                "calibration": "",
                "mask": "",
                "image": os.path.expanduser("~"),
                "pattern": "",
                "overlay": "",
                "phase": "",
            }
            self.resetState()
            self.setUp()
            self.config_file_path = saved_config_file_path
            if intermediate_function:
                intermediate_function()

            self.load_configuration()

        delete_if_exists(self.config_file_path)

    def disable_calibration_check(self):
        self.check_calibration = False

    def save_configuration(self):
        QtWidgets.QFileDialog.getSaveFileName = MagicMock(
            return_value=self.config_file_path
        )
        click_button(self.widget.save_btn)
        self.assertTrue(os.path.isfile(self.config_file_path))

    def load_configuration(self):
        self.model.working_directories = {
            "calibration": "moo",
            "mask": "baa",
            "image": "",
            "pattern": "",
        }
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(
            return_value=self.config_file_path
        )
        click_button(self.widget.load_btn)
        saved_working_directories = self.model.working_directories
        self.assertDictEqual(saved_working_directories, working_directories)
        if self.check_calibration:
            saved_pyfai_params, _ = (
                self.model.calibration_model.get_calibration_parameter()
            )
            for key in ("splineFile", "splinefile"):
                saved_pyfai_params.pop(key, None)
            if "max_shape" in saved_pyfai_params:
                del saved_pyfai_params["max_shape"]
            self.assertDictEqual(saved_pyfai_params, pyfai_params)

    ####################################################################################################################
    def test_with_auto_processing(self):
        self.save_and_load_configuration(self.auto_process_settings)
        self.assertEqual(
            self.model.current_configuration.auto_save_integrated_pattern,
            auto_save_integrated_patterns,
        )
        self.assertEqual(
            self.model.current_configuration.integrated_patterns_file_formats,
            integrated_patterns_file_formats,
        )
        self.assertEqual(
            self.model.current_configuration.img_model.autoprocess, img_autoprocess
        )

    def auto_process_settings(self):
        self.model.current_configuration.auto_save_integrated_pattern = (
            auto_save_integrated_patterns
        )
        self.model.current_configuration.integrated_patterns_file_formats = (
            integrated_patterns_file_formats
        )
        self.model.current_configuration.img_model.autoprocess = img_autoprocess

    ####################################################################################################################
    def test_with_mask(self):
        self.save_and_load_configuration(self.mask_settings)
        self.assertEqual(self.model.use_mask, use_mask)
        self.assertEqual(self.model.transparent_mask, transparent_mask)
        self.assertTrue(np.array_equal(self.model.mask_model.get_img(), self.mask_data))

    def mask_settings(self):
        self.model.current_configuration.use_mask = True
        self.model.current_configuration.transparent_mask = True
        self.mask_data = np.eye(
            self.raw_img_data.shape[0], self.raw_img_data.shape[1], dtype=bool
        )
        self.model.mask_model.set_mask(self.mask_data)

    ####################################################################################################################
    def test_image_rotation(self):
        img_filename = os.path.join(data_path, "CeO2_Pilatus1M.tif")
        with patch.object(
            CalibrationModel,
            "integrate_1d",
            return_value=(np.linspace(0, 20, 1001), np.ones((1001,))),
        ):
            self.load_image(img_filename)
        self.save_and_load_configuration(self.image_transformations)

        img_data = fabio.open(img_filename).data[::-1]
        img_data = rotate_matrix_m90(img_data)

        self.assertNotEqual(
            img_data.shape, self.model.img_model.untransformed_raw_img_data.shape
        )
        self.assertEqual(np.sum(img_data - self.model.img_model.img_data), 0)

    def rotate_image_p90(self):
        click_button(self.widget.calibration_widget.rotate_p90_btn)

    ####################################################################################################################
    def test_with_image_transformations(self):
        img_filename = os.path.join(data_path, "CeO2_Pilatus1M.tif")
        with patch.object(
            CalibrationModel,
            "integrate_1d",
            return_value=(np.linspace(0, 20, 1001), np.ones((1001,))),
        ):
            self.load_image(img_filename)
        self.save_and_load_configuration(self.image_transformations)

        img_data = fabio.open(img_filename).data[::-1]
        img_data = rotate_matrix_m90(img_data)
        img_data = np.fliplr(img_data)
        img_data = rotate_matrix_p90(img_data)
        img_data = np.flipud(img_data)
        img_data = rotate_matrix_m90(img_data)
        img_data = rotate_matrix_m90(img_data)
        img_data = np.fliplr(img_data)
        img_data = np.flipud(img_data)
        img_data = rotate_matrix_p90(img_data)

        self.assertEqual(np.sum(img_data - self.model.img_data), 0)

    def image_transformations(self):
        click_button(self.widget.calibration_widget.rotate_m90_btn)
        click_button(self.widget.calibration_widget.invert_horizontal_btn)
        click_button(self.widget.calibration_widget.rotate_p90_btn)
        click_button(self.widget.calibration_widget.invert_vertical_btn)
        click_button(self.widget.calibration_widget.rotate_m90_btn)
        click_button(self.widget.calibration_widget.rotate_m90_btn)
        click_button(self.widget.calibration_widget.invert_horizontal_btn)
        click_button(self.widget.calibration_widget.invert_vertical_btn)
        click_button(self.widget.calibration_widget.rotate_p90_btn)

    ####################################################################################################################
    def test_with_phases(self):
        self.save_and_load_configuration(self.phase_settings)
        self.assertEqual(self.model.phase_model.phases[0].params["pressure"], pressure)

    def phase_settings(self):
        self.load_phase("ar.jcpds")
        self.model.phase_model.phases[0].params["pressure"] = pressure

    def load_phase(self, filename):
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
            return_value=[os.path.join(jcpds_path, filename)]
        )
        click_button(self.controller.widget.integration_widget.phase_add_btn)

    ####################################################################################################################
    def test_with_overlays(self):
        self.save_and_load_configuration(self.overlay_settings)

    def overlay_settings(self):
        self.controller.integration_controller.widget.qa_set_as_overlay_btn.click()
        self.controller.integration_controller.widget.qa_set_as_overlay_btn.click()
        self.controller.integration_controller.widget.overlay_set_as_bkg_btn.click()
        self.current_pattern_x, self.current_pattern_y = (
            self.model.current_configuration.pattern_model.get_pattern().data
        )

    ####################################################################################################################
    def test_with_roi(self):
        self.save_and_load_configuration(self.roi_settings)
        self.assertTupleEqual(self.model.current_configuration.roi, roi)
        self.assertTrue(
            self.controller.integration_controller.widget.img_roi_btn.isChecked()
        )

    def roi_settings(self):
        self.controller.integration_controller.image_controller.widget.img_roi_btn.click()
        self.model.current_configuration.roi = roi

    ####################################################################################################################
    def test_with_cbn_correction(self):
        self.save_and_load_configuration(
            self.cbn_correction_settings, mock_1d_integration=False
        )
        print(self.model.current_configuration.img_model.img_corrections.corrections)
        self.assertEqual(
            self.model.current_configuration.img_model.img_corrections.corrections[
                "cbn"
            ]._diamond_thickness,
            1.9,
        )

    def cbn_correction_settings(self):
        self.controller.widget.integration_widget.cbn_groupbox.setChecked(True)
        self.controller.widget.integration_widget.cbn_param_tw.cellWidget(0, 1).setText(
            "1.9"
        )
        self.controller.integration_controller.correction_controller.cbn_groupbox_changed()

    ####################################################################################################################
    def test_with_oiadac_correction(self):
        self.save_and_load_configuration(
            self.oiadac_correction_settings, mock_1d_integration=False
        )
        self.assertEqual(
            self.model.current_configuration.img_model.img_corrections.corrections[
                "oiadac"
            ].detector_thickness,
            30,
        )
        self.assertEqual(
            self.model.current_configuration.img_model.img_corrections.corrections[
                "oiadac"
            ].absorption_length,
            450,
        )

    def oiadac_correction_settings(self):
        self.controller.widget.integration_widget.oiadac_groupbox.setChecked(True)
        self.controller.widget.integration_widget.oiadac_param_tw.cellWidget(
            0, 1
        ).setText("30")
        self.controller.widget.integration_widget.oiadac_param_tw.cellWidget(
            1, 1
        ).setText("450")
        self.controller.integration_controller.correction_controller.oiadac_groupbox_changed()

    ####################################################################################################################
    def test_with_transfer_correction(self):
        self.save_and_load_configuration(self.transfer_correction_settings)

        # test model
        self.assertEqual(
            self.model.img_model.transfer_correction.original_filename,
            self.original_filename,
        )
        self.assertEqual(
            self.model.img_model.transfer_correction.response_filename,
            self.response_filename,
        )
        self.assertTrue(self.model.img_model.has_corrections())

        # test widget
        correction_widget = (
            self.widget.integration_widget.integration_control_widget.corrections_control_widget
        )
        self.assertTrue(correction_widget.transfer_gb.isChecked())
        self.assertEqual(
            correction_widget.transfer_original_filename_lbl.text(),
            os.path.basename(self.original_filename),
        )
        self.assertEqual(
            correction_widget.transfer_original_filename_lbl.text(),
            os.path.basename(self.original_filename),
        )

    def transfer_correction_settings(self):
        self.model.calibration_model.detector_reset.emit = MagicMock()
        self.original_filename = os.path.join(
            data_path, "TransferCorrection", "original.tif"
        )
        self.response_filename = os.path.join(
            data_path, "TransferCorrection", "response.tif"
        )
        correction_widget = (
            self.widget.integration_widget.integration_control_widget.corrections_control_widget
        )

        correction_widget.transfer_gb.setChecked(True)
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
            return_value=[self.original_filename]
        )
        click_button(self.widget.integration_widget.load_img_btn)
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(
            return_value=self.original_filename
        )
        click_button(correction_widget.transfer_load_original_btn)
        QtWidgets.QFileDialog.getOpenFileName = MagicMock(
            return_value=self.response_filename
        )
        click_button(correction_widget.transfer_load_response_btn)

    ####################################################################################################################
    def test_configuration_in_cake_mode(self):
        self.save_and_load_configuration(self.cake_settings)
        self.assertTrue(self.model.current_configuration.auto_integrate_cake)

    def cake_settings(self):
        self.controller.integration_controller.widget.img_mode_btn.click()

    ####################################################################################################################
    def test_with_fit_bg(self):
        self.save_and_load_configuration(self.fit_bg_settings)
        self.assertEqual(
            self.widget.integration_widget.bkg_pattern_poly_order_sb.value(), poly_order
        )
        self.assertTrue(self.widget.integration_widget.qa_bkg_pattern_btn.isChecked())
        click_button(self.widget.integration_mode_btn)
        self.widget.show()
        self.assertTrue(
            self.widget.integration_widget.qa_bkg_pattern_inspect_btn.isVisible()
        )
        self.widget.close()

    def test_with_q_and_fit_bg(self):
        self.save_and_load_configuration(self.q_and_fit_bg_settings)
        self.assertAlmostEqual(
            float(
                self.controller.integration_controller.widget.bkg_pattern_x_min_txt.text()
            ),
            x_min,
            1,
        )
        self.assertAlmostEqual(
            float(
                self.controller.integration_controller.widget.bkg_pattern_x_max_txt.text()
            ),
            x_max,
            1,
        )

    ####################################################################################################################
    def test_multiple_configurations(self):
        self.save_and_load_configuration(self.add_configuration)
        self.assertEqual(len(self.config_widget.configuration_btns), 2)

    def add_configuration(self):
        click_button(self.widget.configuration_widget.add_configuration_btn)

    def fit_bg_settings(self):
        click_button(self.controller.integration_controller.widget.qa_bkg_pattern_btn)
        self.controller.integration_controller.widget.bkg_pattern_poly_order_sb.setValue(
            poly_order
        )

    def q_and_fit_bg_settings(self):
        self.controller.integration_controller.widget.pattern_q_btn.click()
        self.controller.integration_controller.widget.qa_bkg_pattern_btn.click()
        self.controller.integration_controller.widget.bkg_pattern_poly_order_sb.setValue(
            poly_order
        )

        enter_value_into_text_field(
            self.controller.integration_controller.widget.bkg_pattern_x_min_txt,
            str(x_min),
        )
        enter_value_into_text_field(
            self.controller.integration_controller.widget.bkg_pattern_x_max_txt,
            str(x_max),
        )

    ####################################################################################################################
    def test_with_background_image(self):
        self.save_and_load_configuration(self.add_background_image)
        self.assertTrue(self.model.img_model.has_background())
        self.assertEqual(test_image_file_name, self.model.img_model.background_filename)
        self.assertEqual(
            os.path.split(test_image_file_name)[1],
            self.widget.integration_widget.bkg_image_filename_lbl.text(),
        )

    def add_background_image(self):
        self.model.img_model.load_background(test_image_file_name)

    ####################################################################################################################
    def test_with_automatic_background_subtraction(self):
        self.save_and_load_configuration(
            self.activate_automatic_background_subtraction, mock_1d_integration=True
        )
        self.assertAlmostEqual(self.model.pattern.auto_bkg_roi[0], 9.0)
        self.assertTrue(self.widget.integration_widget.qa_bkg_pattern_btn.isChecked())
        self.assertAlmostEqual(
            float(self.widget.integration_widget.bkg_pattern_x_min_txt.text()), 9
        )
        self.assertTrue(self.widget.integration_widget.qa_bkg_pattern_btn.isChecked())

        click_button(self.widget.integration_mode_btn)
        self.widget.show()
        self.assertTrue(
            self.widget.integration_widget.qa_bkg_pattern_inspect_btn.isVisible()
        )
        self.widget.close()

    def activate_automatic_background_subtraction(self):
        self.model.pattern.load(os.path.join(data_path, "pattern_001.xy"))
        click_button(self.widget.integration_widget.qa_bkg_pattern_btn)
        enter_value_into_text_field(
            self.widget.integration_widget.bkg_pattern_x_min_txt, "9"
        )
        print(self.model.pattern)
        print(type(self.model.pattern))
        self.assertAlmostEqual(self.model.pattern.auto_bkg_roi[0], 9)

    ####################################################################################################################
    def test_save_settings_on_closing(self):
        with patch.object(
            CalibrationModel,
            "integrate_1d",
            return_value=(np.linspace(0, 20, 1001), np.ones((1001,))),
        ):
            self.load_image(test_image_file_name)
            self.controller.widget.close()
            self.controller = MainController(
                use_settings=True, settings_directory=data_path
            )
            self.assertEqual(self.model.img_model.filename, test_image_file_name)

    ####################################################################################################################
    def test_file_browsing(self):
        self.save_and_load_configuration(self.prepare_file_browsing)
        with patch.object(
            CalibrationModel,
            "integrate_1d",
            return_value=(np.linspace(0, 20, 1001), np.ones((1001,))),
        ):
            click_button(self.widget.integration_widget.img_step_file_widget.next_btn)
            self.assertEqual(
                str(self.widget.integration_widget.img_filename_txt.text()),
                "image_002.tif",
            )

    def prepare_file_browsing(self):
        self.model.calibration_model.detector_reset.emit = MagicMock()
        self.load_image(os.path.join(data_path, "image_001.tif"))

    ####################################################################################################################
    def test_distortion_correction(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_distortion_correction_test,
            intermediate_function=self.disable_calibration_check,
        )
        self.assertIsNotNone(self.model.calibration_model.distortion_spline_filename)
        self.assertEqual(
            self.widget.calibration_widget.spline_filename_txt.text(), "f4mnew.spline"
        )

    def prepare_distortion_correction_test(self):
        self.model.calibration_model.detector_reset.emit = MagicMock()
        self.model.img_model.load(
            os.path.join(data_path, "distortion", "CeO2_calib.edf")
        )

        self.model.calibration_model.find_peaks_automatic(1025.1, 1226.8, 0)
        self.model.calibration_model.set_calibrant(
            os.path.join(calibrants_path, "CeO2.D")
        )
        self.model.calibration_model.start_values["dist"] = 300e-3
        self.model.calibration_model.start_values["pixel_height"] = 50e-6
        self.model.calibration_model.start_values["pixel_width"] = 50e-6
        self.model.calibration_model.start_values["wavelength"] = 0.1e-10

        self.model.calibration_model.load_distortion(
            os.path.join(data_path, "distortion", "f4mnew.spline")
        )
        self.model.calibration_model.calibrate()

        _, y1 = self.model.calibration_model.integrate_1d()

    ####################################################################################################################
    def test_series_loading(self):
        from dioptas.model.loader.KaraboLoader import extra_data_installed

        if not extra_data_installed:
            return
        self.check_calibration = False
        self.save_and_load_configuration(self.prepare_series_file)
        self.assertTrue(self.model.img_model.series_max > 1)

    def prepare_series_file(self):
        self.model.img_model.load(os.path.join(data_path, "karabo_epix.h5"))
        self.assertTrue(self.model.img_model.series_max > 1)

    ####################################################################################################################
    def test_using_predefined_detector(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_load_predefined_detector_test,
            intermediate_function=self.disable_calibration_check,
        )
        from pyFAI import detectors

        self.assertIsInstance(
            self.model.calibration_model.detector, detectors.PilatusCdTe1M
        )
        self.assertEqual(self.model.calibration_model.orig_pixel1, 172e-6)
        self.assertEqual(self.model.calibration_model.orig_pixel2, 172e-6)

        detector_gb = (
            self.widget.calibration_widget.calibration_control_widget.calibration_parameters_widget.detector_gb
        )
        self.assertEqual(
            self.widget.calibration_widget.detectors_cb.currentText(), "Pilatus CdTe 1M"
        )
        self.assertEqual(float(detector_gb.pixel_width_txt.text()), 172)
        self.assertEqual(float(detector_gb.pixel_height_txt.text()), 172)

        self.assertFalse(detector_gb.pixel_width_txt.isEnabled())
        self.assertFalse(detector_gb.pixel_height_txt.isEnabled())

    def prepare_load_predefined_detector_test(self):
        self.widget.calibration_widget.detectors_cb.setCurrentIndex(
            self.widget.calibration_widget.detectors_cb.findText("Pilatus CdTe 1M")
        )

    ###################################################################################################################
    def test_using_loaded_nexus_detector(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_using_loaded_nexus_detector_test,
            intermediate_function=self.disable_calibration_check,
        )

        from ...model.CalibrationModel import DetectorModes
        from pyFAI import detectors

        self.assertEqual(
            self.model.calibration_model.detector_mode, DetectorModes.NEXUS
        )
        self.assertIsInstance(
            self.model.calibration_model.detector, detectors.NexusDetector
        )

    def prepare_using_loaded_nexus_detector_test(self):
        self.model.img_model._img_data = np.ones((1048, 1032))
        self.model.calibration_model.load_detector_from_file(
            os.path.join(data_path, "detector.h5")
        )

    ###################################################################################################
    def test_using_predefined_detector_and_rotate(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_using_predefined_detector_and_rotate,
            intermediate_function=self.disable_calibration_check,
        )
        self.assertEqual(self.model.calibration_model.detector.shape, (981, 1043))

    def prepare_using_predefined_detector_and_rotate(self):
        self.widget.calibration_widget.detectors_cb.setCurrentIndex(
            self.widget.calibration_widget.detectors_cb.findText("Pilatus CdTe 1M")
        )
        click_button(self.widget.calibration_widget.rotate_p90_btn)
        self.assertEqual(self.model.calibration_model.detector.shape, (981, 1043))

    ###################################################################################################
    def test_using_detector_and_calibration(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_using_detector_and_calibration,
            intermediate_function=self.disable_calibration_check,
        )

        self.assertAlmostEqual(self.model.calibration_model.detector.pixel1, 172e-6)

    def prepare_using_detector_and_calibration(self):
        self.model.calibration_model.detector_reset.emit = MagicMock()
        self.model.calibration_model.load(
            os.path.join(data_path, "CeO2_Pilatus1M.poni")
        )
        self.model.img_model.load(os.path.join(data_path, "image_001.tif"))

        self.assertAlmostEqual(self.model.calibration_model.detector.pixel1, 172e-6)

    ###################################################################################################
    def test_saving_poni_detector_orientation(self):
        self.check_calibration = False
        self.save_and_load_configuration(
            self.prepare_saving_poni_detector_orientation,
            intermediate_function=self.disable_calibration_check,
        )

        assert self.model.calibration_model.polarization_factor == -0.1
        assert self.model.calibration_model.detector.pixel1 == 172e-6
        assert (
            self.model.calibration_model.pattern_geometry.get_config()
            == self.model.calibration_model.cake_geometry.get_config()
        )

    def prepare_saving_poni_detector_orientation(self):
        self.model.calibration_model.load(
            os.path.join(data_path, "CeO2_Pilatus1M.poni")
        )
        self.model.calibration_model.polarization_factor = -0.1



# SPDX-License-Identifier: MIT

import os
import numpy as np
import pytest
from unittest.mock import MagicMock

from ...model.OverlayModel import OverlayModel

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, "../data")


@pytest.fixture
def overlay_model():
    return OverlayModel()


def test_add_overlay(overlay_model: OverlayModel):
    x_overlay = np.linspace(0, 10)
    y_overlay = np.linspace(0, 100)
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy")

    assert len(overlay_model.overlays) == 1

    new_overlay = overlay_model.get_overlay(0)
    assert new_overlay is not None
    assert new_overlay.name == "dummy"
    assert np.array_equal(new_overlay.x, x_overlay)
    assert np.array_equal(new_overlay.y, y_overlay)


def test_add_overlay_from_file(overlay_model: OverlayModel):
    filename = os.path.join(data_path, "pattern_001.xy")
    overlay_model.add_overlay_file(filename)

    assert len(overlay_model.overlays) == 1
    overlay = overlay_model.get_overlay(0)
    assert overlay is not None
    assert overlay.name == "".join(os.path.basename(filename).split(".")[0:-1])


def test_different_colors_for_overlay(overlay_model: OverlayModel):
    x_overlay = np.linspace(0, 10)
    y_overlay = np.linspace(0, 100)
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy")

    assert len(overlay_model.overlays) == 3

    color1 = overlay_model.get_overlay_color(0)
    color2 = overlay_model.get_overlay_color(1)
    color3 = overlay_model.get_overlay_color(2)

    assert not np.array_equal(color1, color2)
    assert not np.array_equal(color2, color3)
    assert not np.array_equal(color1, color3)


def test_move_up_overlay(overlay_model: OverlayModel):
    x_overlay = np.linspace(0, 10)
    y_overlay = np.linspace(0, 100)
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy1")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy2")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy3")

    overlay_model.move_overlay_up(1)

    overlay1 = overlay_model.get_overlay(0)
    overlay2 = overlay_model.get_overlay(1)
    overlay3 = overlay_model.get_overlay(2)

    assert overlay1 is not None
    assert overlay2 is not None
    assert overlay3 is not None

    assert overlay1.name == "dummy2"
    assert overlay2.name == "dummy1"
    assert overlay3.name == "dummy3"


def test_move_down_overlay(overlay_model: OverlayModel):
    x_overlay = np.linspace(0, 10)
    y_overlay = np.linspace(0, 100)
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy1")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy2")
    overlay_model.add_overlay(x_overlay, y_overlay, "dummy3")

    overlay_model.move_overlay_down(1)

    overlay1 = overlay_model.get_overlay(0)
    overlay2 = overlay_model.get_overlay(1)
    overlay3 = overlay_model.get_overlay(2)

    assert overlay1 is not None
    assert overlay2 is not None
    assert overlay3 is not None

    assert overlay1.name == "dummy1"
    assert overlay2.name == "dummy3"
    assert overlay3.name == "dummy2"


def test_move_signals(overlay_model: OverlayModel):
    for i in range(10):
        overlay_model.add_overlay(np.linspace(0, 10), np.linspace(0, 100), f"dummy{i}")

    overlay_model.overlay_changed.emit = MagicMock()
    overlay_model.move_overlay_up(1)

    assert overlay_model.overlay_changed.emit.call_count == 2
    overlay_model.overlay_changed.emit.assert_any_call(1)
    overlay_model.overlay_changed.emit.assert_any_call(0)

    overlay_model.overlay_changed.emit.reset_mock()
    overlay_model.move_overlay_down(3)
    assert overlay_model.overlay_changed.emit.call_count == 2
    overlay_model.overlay_changed.emit.assert_any_call(3)
    overlay_model.overlay_changed.emit.assert_any_call(4)



# SPDX-License-Identifier: MIT

import os
import numpy as np
from pytest import approx

from qtpy import QtCore, QtWidgets
from mock import MagicMock
from ..utility import click_button

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, os.pardir, "data")


def mock_open_filename(filepath):
    QtWidgets.QFileDialog.getOpenFileNames = MagicMock(return_value=[filepath])
    QtWidgets.QFileDialog.getOpenFileName = MagicMock(return_value=filepath)


def test_synchronization_of_view_range(main_controller):
    # Herbert opens the loads an image and zooms into a certain area,
    # then he switches to mask mode and sees the same area in the mask view.
    mock_open_filename(os.path.join(data_path, "CeO2_Pilatus1M.tif"))

    click_button(main_controller.widget.calibration_mode_btn)
    click_button(main_controller.widget.calibration_widget.load_img_btn)

    calibration_img_vb = (
        main_controller.widget.calibration_widget.img_widget.img_view_box
    )
    mask_img_vb = main_controller.widget.mask_widget.img_widget.img_view_box
    integration_img_vb = (
        main_controller.widget.integration_widget.img_widget.img_view_box
    )
    map_img_vb = main_controller.widget.map_widget.img_plot_widget.img_view_box

    calibration_img_vb.setRange(QtCore.QRectF(-10, -10, 20, 20))
    click_button(main_controller.widget.mask_mode_btn)

    range_diff = np.sum(
        np.array(calibration_img_vb.targetRange()) - np.array(mask_img_vb.targetRange())
    )

    assert range_diff == approx(0)

    # Herbert zooms into a different area in the mask view and sees the same area in the calibration view.

    mask_img_vb.setRange(QtCore.QRectF(100, 100, 300, 300))
    click_button(main_controller.widget.calibration_mode_btn)

    range_diff = np.sum(
        np.array(calibration_img_vb.targetRange()) - np.array(mask_img_vb.targetRange())
    )

    assert range_diff == approx(0)

    # HE also goes to the integration view and observes the same a similar range there.
    click_button(main_controller.widget.integration_mode_btn)

    range_diff = np.sum(
        np.array(calibration_img_vb.targetRange())
        - np.array(integration_img_vb.targetRange())
    )

    assert range_diff == approx(0)

    # then he checks the map view and see that the range is also the same there.
    click_button(main_controller.widget.map_mode_btn)
    range_diff = np.sum(
        np.array(calibration_img_vb.targetRange()) - np.array(map_img_vb.targetRange())
    )

    assert range_diff == approx(0)

    # He zooms out of the image in the map view and sees that the range is also the same in the other views.
    map_img_vb.setRange(QtCore.QRectF(-10, -10, 20, 20))

    click_button(main_controller.widget.calibration_mode_btn)
    range_diff = np.sum(
        np.array(calibration_img_vb.targetRange()) - np.array(map_img_vb.targetRange())
    )

    assert range_diff == approx(0)


def test_remove_img_background_in_view(main_controller):
    # Herbert opens an image in the integration view and wants to remove the background signal.
    mock_open_filename(os.path.join(data_path, "image_001.tif"))
    click_button(main_controller.widget.integration_mode_btn)
    click_button(main_controller.widget.integration_widget.load_img_btn)

    integration_widget = main_controller.widget.integration_widget
    image_widget = integration_widget.img_widget

    data_before_bg = image_widget.data_img_item.image.copy()

    # He clicks the background subtraction button and sees that the image does not change and wonders why.
    integration_widget.integration_control_widget.tab_widget_1.setCurrentIndex(5)
    mock_open_filename(os.path.join(data_path, "image_002.tif"))
    click_button(
        integration_widget.integration_control_widget.background_control_widget.load_image_btn
    )
    assert (
        integration_widget.integration_control_widget.background_control_widget.filename_lbl.text()
        == "image_002.tif"
    )

    data_after_bg = image_widget.data_img_item.image.copy()
    assert np.array_equal(data_before_bg, data_after_bg)

    # He realizes that there is s a show background subtraction button and clicks it.
    # With satisfaction he sees the background subtracted image
    click_button(
        integration_widget.integration_image_widget.show_background_subtracted_img_btn
    )
    assert (
        integration_widget.integration_image_widget.show_background_subtracted_img_btn.isChecked()
    )
    data_before_bg = image_widget.data_img_item.image.copy()
    assert not np.array_equal(data_before_bg, data_after_bg)



# SPDX-License-Identifier: MIT

import os
from mock import MagicMock
import gc

import numpy as np

from ..utility import (
    QtTest,
    delete_if_exists,
    click_button,
    enter_value_into_text_field,
)

from qtpy import QtWidgets
from xypattern import Pattern

from ...controller.ConfigurationController import ConfigurationController
from ...model.DioptasModel import DioptasModel
from ...widgets.ConfigurationWidget import ConfigurationWidget

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, "../data")
jcpds_path = os.path.join(data_path, "jcpds")


class ConfigurationControllerTest(QtTest):
    def setUp(self):
        self.config_widget = ConfigurationWidget()
        self.model = DioptasModel()
        self.config_controller = ConfigurationController(
            configuration_widget=self.config_widget,
            dioptas_model=self.model,
            controllers=[],
        )

    def tearDown(self):
        delete_if_exists(os.path.join(data_path, "combined_pattern.xy"))
        del self.model
        del self.config_widget
        del self.config_controller
        gc.collect()

    def test_initial_configuration_display(self):
        self.assertEqual(len(self.config_widget.configuration_btns), 1)

    def test_adding_configurations(self):
        click_button(self.config_widget.add_configuration_btn)
        self.assertEqual(len(self.model.configurations), 2)
        self.assertEqual(len(self.config_widget.configuration_btns), 2)
        self.assertTrue(self.config_widget.configuration_btns[1].isChecked())

        click_button(self.config_widget.add_configuration_btn)
        self.assertEqual(len(self.model.configurations), 3)
        self.assertEqual(len(self.config_widget.configuration_btns), 3)
        self.assertTrue(self.config_widget.configuration_btns[2].isChecked())

        self.assertEqual(self.config_widget.configuration_btns[0].text(), "1")
        self.assertEqual(self.config_widget.configuration_btns[1].text(), "2")
        self.assertEqual(self.config_widget.configuration_btns[2].text(), "3")

    def test_selecting_configuration(self):
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)

        self.assertEqual(self.model.configuration_ind, 3)
        self.assertTrue(self.config_widget.configuration_btns[-1].isChecked())

        click_button(self.config_widget.configuration_btns[0])
        self.assertEqual(self.model.configuration_ind, 0)

    def test_remove_last_configuration(self):
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)

        self.assertEqual(self.model.configuration_ind, 3)
        self.assertTrue(self.config_widget.configuration_btns[3].isChecked())

        click_button(self.config_widget.remove_configuration_btn)

        self.assertEqual(self.model.configuration_ind, 2)
        self.assertTrue(self.config_widget.configuration_btns[2].isChecked())
        self.assertEqual(len(self.config_widget.configuration_btn_group.buttons()), 3)

    def test_remove_first_configuration(self):
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)

        click_button(self.config_widget.configuration_btns[0])
        self.assertEqual(self.model.configuration_ind, 0)

        click_button(self.config_widget.remove_configuration_btn)

        self.assertEqual(self.model.configuration_ind, 0)
        self.assertTrue(self.config_widget.configuration_btns[0].isChecked())
        self.assertEqual(len(self.model.configurations), 3)

        self.assertEqual(self.config_widget.configuration_btns[0].text(), "1")
        self.assertEqual(self.config_widget.configuration_btns[1].text(), "2")
        self.assertEqual(self.config_widget.configuration_btns[2].text(), "3")

    def test_remove_in_between_configuration(self):
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)
        click_button(self.config_widget.add_configuration_btn)

        click_button(self.config_widget.configuration_btns[1])
        click_button(self.config_widget.remove_configuration_btn)

        self.assertEqual(self.model.configuration_ind, 1)
        self.assertTrue(self.config_widget.configuration_btns[1].isChecked())
        self.assertEqual(len(self.model.configurations), 3)

        self.assertEqual(self.config_widget.configuration_btns[0].text(), "1")
        self.assertEqual(self.config_widget.configuration_btns[1].text(), "2")
        self.assertEqual(self.config_widget.configuration_btns[2].text(), "3")

    def test_using_factors(self):
        self.model.img_model.load(os.path.join(data_path, "image_001.tif"))
        data1 = np.copy(self.model.img_data)
        enter_value_into_text_field(self.config_widget.factor_txt, 2.5)
        self.assertTrue(np.array_equal(2.5 * data1, self.model.img_data))

        self.model.add_configuration()
        self.assertEqual(float(str(self.config_widget.factor_txt.text())), 1.0)
        enter_value_into_text_field(self.config_widget.factor_txt, 3.5)
        self.assertEqual(self.model.img_model.factor, 3.5)

        self.model.select_configuration(0)
        self.assertEqual(self.model.img_model.factor, 2.5)
        self.assertEqual(float(str(self.config_widget.factor_txt.text())), 2.5)

    def test_file_browsing(self):
        self.model.img_model.load(os.path.join(data_path, "image_001.tif"))
        self.model.add_configuration()
        self.model.img_model.load(os.path.join(data_path, "image_001.tif"))

        self.config_widget.file_iterator_pos_txt.setText("0")

        click_button(self.config_widget.next_file_btn)

        self.assertEqual(
            os.path.abspath(self.model.configurations[0].img_model.filename),
            os.path.abspath(os.path.join(data_path, "image_002.tif")),
        )

        self.assertEqual(
            self.model.configurations[1].img_model.filename,
            os.path.abspath(os.path.join(data_path, "image_002.tif")),
        )

        click_button(self.config_widget.previous_file_btn)

        self.assertEqual(
            os.path.abspath(self.model.configurations[0].img_model.filename),
            os.path.abspath(os.path.join(data_path, "image_001.tif")),
        )

        self.assertEqual(
            self.model.configurations[1].img_model.filename,
            os.path.abspath(os.path.join(data_path, "image_001.tif")),
        )

    def test_folder_browsing(self):
        self.model.img_model.load(
            os.path.join(data_path, "FileIterator", "run1", "image_1.tif")
        )
        self.model.add_configuration()
        self.model.img_model.load(
            os.path.join(data_path, "FileIterator", "run1", "image_1.tif")
        )

        click_button(self.config_widget.next_folder_btn)

        self.assertEqual(
            self.model.configurations[0].img_model.filename,
            os.path.abspath(
                os.path.join(data_path, "FileIterator", "run2", "image_1.tif")
            ),
        )

        self.assertEqual(
            self.model.configurations[1].img_model.filename,
            os.path.abspath(
                os.path.join(data_path, "FileIterator", "run2", "image_1.tif")
            ),
        )

        click_button(self.config_widget.previous_folder_btn)

        self.assertEqual(
            self.model.configurations[0].img_model.filename,
            os.path.abspath(
                os.path.join(data_path, "FileIterator", "run1", "image_1.tif")
            ),
        )

        self.assertEqual(
            self.model.configurations[1].img_model.filename,
            os.path.abspath(
                os.path.join(data_path, "FileIterator", "run1", "image_1.tif")
            ),
        )

    def test_save_combined_pattern(self):
        # prepare two patterns
        x1 = np.linspace(0, 10)
        y1 = np.ones(x1.shape)
        pattern1 = Pattern(x1, y1)

        x2 = np.linspace(7, 15)
        y2 = np.ones(x2.shape) * 2
        pattern2 = Pattern(x2, y2)

        self.model.pattern_model.pattern = pattern1
        self.model.add_configuration()
        self.model.pattern_model.pattern = pattern2

        self.model.combine_patterns = True

        # click the button
        file_path = os.path.join(data_path, "combined_pattern.xy")
        QtWidgets.QFileDialog.getSaveFileName = MagicMock(return_value=file_path)
        click_button(self.config_widget.saved_combined_patterns_btn)

        # load and check that it worked
        saved_pattern = Pattern.from_file(file_path)
        x3, _ = saved_pattern.data
        self.assertLess(np.min(x3), 7)
        self.assertGreater(np.max(x3), 10)



# SPDX-License-Identifier: MIT


from dioptas.model.OverlayModel import OverlayModel
from dioptas.widgets.plot_widgets import PatternWidget


class OverlayInPatternController(object):
    def __init__(self, pattern_widget: PatternWidget, overlay_model: OverlayModel):
        self.model = overlay_model
        self.pattern_widget = pattern_widget

        self.connect()

    def connect(self):
        self.model.overlay_added.connect(self.overlay_added)
        self.model.overlay_removed.connect(self.overlay_removed)
        self.model.overlay_changed.connect(self.overlay_changed)

    def overlay_added(self):
        overlay = self.model.get_overlay(len(self.model.overlays) - 1)
        color = self.model.get_overlay_color(len(self.model.overlays) - 1)
        if overlay is not None:
            self.pattern_widget.add_overlay(overlay, color)

    def overlay_removed(self, index: int):
        self.pattern_widget.remove_overlay(index)

    def overlay_changed(self, index: int):
        overlay = self.model.get_overlay(index)
        if overlay is not None:
            self.pattern_widget.update_overlay(index, overlay)



# SPDX-License-Identifier: MIT

from qtpy import QtWidgets

from ...CustomWidgets import LabelAlignRight, HorizontalSpacerItem
from ..CustomWidgets import MouseCurrentAndClickedWidget, MouseUnitCurrentAndClickedWidget
from .. import CLICKED_COLOR


class IntegrationStatusWidget(QtWidgets.QWidget):
    def __init__(self):
        super(IntegrationStatusWidget, self).__init__()

        self._layout = QtWidgets.QHBoxLayout()
        self._layout.setContentsMargins(6, 3, 6, 0)
        self._layout.setSpacing(6)

        self.mouse_pos_widget = MouseCurrentAndClickedWidget(CLICKED_COLOR)
        self.mouse_unit_widget = MouseUnitCurrentAndClickedWidget(CLICKED_COLOR)
        self.bkg_name_lbl = LabelAlignRight('')
        self.change_view_btn = QtWidgets.QPushButton('Change View')

        self._layout.addWidget(self.change_view_btn)
        self._layout.addWidget(self.mouse_pos_widget)
        self._layout.addSpacerItem(HorizontalSpacerItem())
        self._layout.addWidget(self.mouse_unit_widget)
        self._layout.addSpacerItem(HorizontalSpacerItem())
        self._layout.addWidget(self.bkg_name_lbl)

        self.setLayout(self._layout)



import numpy as np

from ...model.util.calc import trim_trailing_zeros


def test_trim_trailing_zeros():
    x = np.linspace(0, 20)
    y = np.ones_like(x)
    y[-10:] = 0

    x_trim, y_trim = trim_trailing_zeros(x, y)

    assert len(y_trim) == len(y) - 10
    assert len(x_trim) == len(y) - 10



import os
from dioptas.model.Configuration import Configuration
from ..utility import unittest_data_path


def test_auto_save_patterns(tmp_path):
    config = Configuration()
    config.calibration_model.is_calibrated = True
    config.auto_save_integrated_pattern = True
    config.working_directories["pattern"] = tmp_path
    config.img_model.load(os.path.join(unittest_data_path, "image_001.tif"))
    config.integrate_image_1d()

    assert os.path.exists(os.path.join(tmp_path, "image_001.xy"))


def test_auto_save_background_subtracted_pattern(tmp_path):
    config = Configuration()
    config.calibration_model.is_calibrated = True
    config.auto_save_integrated_pattern = True
    config.working_directories["pattern"] = tmp_path
    config.img_model.load(os.path.join(unittest_data_path, "image_001.tif"))
    config.pattern_model.set_auto_background_subtraction([2, 50, 50])
    config.integrate_image_1d()

    assert os.path.exists(os.path.join(tmp_path, "image_001.xy"))
    assert os.path.exists(os.path.join(tmp_path, "bkg_subtracted", "image_001.xy"))




# SPDX-License-Identifier: MIT

import os

from qtpy import QtWidgets, QtGui, QtCore

from .ConfigurationWidget import ConfigurationWidget
from .CalibrationWidget import CalibrationWidget
from .MaskWidget import MaskWidget
from .integration import IntegrationWidget
from .MapWidget import MapWidget
from .CustomWidgets import (
    VerticalSpacerItem,
    CheckableFlatButton,
    FlatButton,
    VerticalLine,
    HorizontalLine,
)

from .. import icons_path


class MainWidget(QtWidgets.QWidget):
    def __init__(self, *args, **kwargs):
        super(MainWidget, self).__init__(*args, **kwargs)

        self._create_layouts()

        self.setLayout(self._outer_layout)

        self._create_menu()
        self._create_mode_menu()

        self._left_layout.addLayout(self._menu_layout)
        self._left_layout.addLayout(self._mode_layout)
        self._left_layout.addLayout(self._external_actions_layout)

        self._outer_layout.addLayout(self._left_layout)
        self._outer_layout.addWidget(VerticalLine())
        self._outer_layout.addLayout(self._content_layout)

        self.configuration_widget = ConfigurationWidget(self)
        self.configuration_widget.setVisible(False)
        self._content_layout.addWidget(self.configuration_widget)

        self._create_main_frame()

        self._style_layouts()
        self.style_widgets()
        self.add_menu_popup()
        self.add_tooltips()

        self._content_layout.addWidget(self.main_frame)
        self.setWindowIcon(QtGui.QIcon(os.path.join(icons_path, "icon.svg")))

    def _style_layouts(self):
        self._outer_layout.setContentsMargins(0, 0, 0, 0)
        self._outer_layout.setStretchFactor(self._left_layout, 0)
        self._outer_layout.setStretchFactor(self._content_layout, 100)
        self._outer_layout.setSpacing(0)
        self._mode_layout.setContentsMargins(0, 0, 0, 0)
        self._mode_layout.setSpacing(0)
        self._content_layout.setSpacing(0)
        self._content_layout.setContentsMargins(0, 6, 0, 0)
        self._left_layout.setContentsMargins(0, 0, 0, 0)
        self._left_layout.setSpacing(6)
        self._external_actions_layout.setContentsMargins(6, 6, 6, 6)
        self._external_actions_layout.setSpacing(6)

    def _create_layouts(self):
        self._outer_layout = QtWidgets.QHBoxLayout()
        self._left_layout = QtWidgets.QVBoxLayout()
        self._external_actions_layout = QtWidgets.QVBoxLayout()
        self._content_layout = QtWidgets.QVBoxLayout()
        self._mode_layout = QtWidgets.QVBoxLayout()

    def _create_mode_menu(self):
        self.mode_btn_group = QtWidgets.QButtonGroup()
        self.calibration_mode_btn = CheckableFlatButton("CALIB", self)
        self.calibration_mode_btn.setObjectName("calibration_mode_btn")
        self.calibration_mode_btn.setChecked(True)
        self.mask_mode_btn = CheckableFlatButton("MASK", self)
        self.mask_mode_btn.setObjectName("mask_mode_btn")
        self.integration_mode_btn = CheckableFlatButton("INT", self)
        self.integration_mode_btn.setObjectName("integration_mode_btn")
        self.map_mode_btn = CheckableFlatButton("MAP", self)
        self.map_mode_btn.setObjectName("map_mode_btn")

        self.mode_btn_group.addButton(self.calibration_mode_btn)
        self.mode_btn_group.addButton(self.mask_mode_btn)
        self.mode_btn_group.addButton(self.integration_mode_btn)
        self.mode_btn_group.addButton(self.map_mode_btn)

        self._mode_layout.addWidget(self.calibration_mode_btn)
        self._mode_layout.addWidget(HorizontalLine())
        self._mode_layout.addWidget(self.mask_mode_btn)
        self._mode_layout.addWidget(HorizontalLine())
        self._mode_layout.addWidget(self.integration_mode_btn)
        self._mode_layout.addWidget(HorizontalLine())
        self._mode_layout.addWidget(self.map_mode_btn)
        self._mode_layout.addSpacerItem(VerticalSpacerItem())

    def _create_menu(self):
        self.menu_btn = QtWidgets.QPushButton("...")
        self.menu_btn.setObjectName("menu_btn")

        self.show_configuration_menu_btn = CheckableFlatButton("C")

        self._menu_layout = QtWidgets.QHBoxLayout()
        self._menu_layout.setContentsMargins(6, 6, 3, 0)
        self._menu_layout.setSpacing(12)

        self._menu_layout.addWidget(self.menu_btn)
        self._menu_layout.addWidget(self.show_configuration_menu_btn)

        self.save_btn = FlatButton("Save Project")
        self.load_btn = FlatButton("Open Project")
        self.reset_btn = FlatButton("Reset Project")

    def _create_main_frame(self):
        self.main_frame = QtWidgets.QWidget(self)
        self._layout_main_frame = QtWidgets.QVBoxLayout()
        self._layout_main_frame.setContentsMargins(0, 10, 6, 6)
        self._layout_main_frame.setSpacing(0)
        self.main_frame.setLayout(self._layout_main_frame)

        self.calibration_widget = CalibrationWidget(self)
        self.mask_widget = MaskWidget(self)
        self.integration_widget = IntegrationWidget(self)
        self.map_widget = MapWidget(self)

        self._layout_main_frame.addWidget(self.calibration_widget)
        self._layout_main_frame.addWidget(self.mask_widget)
        self._layout_main_frame.addWidget(self.integration_widget)
        self._layout_main_frame.addWidget(self.map_widget)

        self.mask_widget.setVisible(False)
        self.integration_widget.setVisible(False)
        self.map_widget.setVisible(False)

        self._content_layout.addWidget(self.main_frame)
        self._content_layout.setStretchFactor(self.main_frame, 100)

        self.style_widgets()
        self.add_tooltips()

        self.setWindowIcon(QtGui.QIcon(os.path.join(icons_path, "icon.svg")))

    def style_widgets(self):
        self._style_mode_btns()
        self._style_menu_btn()

        button_height = 24
        button_width = 24
        adjust_height_btns = [
            self.show_configuration_menu_btn,
        ]
        for btn in adjust_height_btns:
            btn.setHeight(button_height)
            btn.setWidth(button_width)

        self.configuration_widget.setMaximumHeight(28)

    def _style_menu_btn(self):
        self.menu_btn.setFixedWidth(30)
        self.menu_btn.setFixedHeight(30)

    def _style_mode_btns(self):
        mode_btn_width = 75
        mode_btn_height = 75
        mode_btns = [
            self.calibration_mode_btn,
            self.mask_mode_btn,
            self.integration_mode_btn,
            self.map_mode_btn,
        ]
        for btn in mode_btns:
            # btn.setCheckable(True)
            btn.setFixedWidth(mode_btn_width)
            btn.setFixedHeight(mode_btn_height)

    def add_menu_popup(self):
        self.menu_btn.clicked.connect(self.show_menu_popup)

    def show_menu_popup(self):
        widget = MenuPopup(self, [self.load_btn, self.save_btn, self.reset_btn])
        btn = self.menu_btn
        widget.adjustSize()
        position = self.mapToGlobal(QtCore.QPoint(btn.x() + btn.width() + 3, btn.y()))
        widget.move(position)
        widget.show()

    def add_tooltips(self):
        self.menu_btn.setToolTip("Project Menu")
        self.show_configuration_menu_btn.setToolTip("Show Configurations")
        self.calibration_mode_btn.setToolTip("Calibration Mode")
        self.mask_mode_btn.setToolTip("Mask Mode")
        self.integration_mode_btn.setToolTip("Integration Mode")

    def create_external_actions(self, quick_actions):
        self.external_action_btns = {}
        for action in quick_actions:
            btn = QtWidgets.QPushButton(action['name'])
            self.external_action_btns[action['name']] = btn
            self._external_actions_layout.addWidget(btn)


class MenuPopup(QtWidgets.QFrame):
    def __init__(self, parent=None, menu_items=None):
        super(MenuPopup, self).__init__(parent)
        self.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.setFrameShadow(QtWidgets.QFrame.Raised)
        self.setLineWidth(1)
        self.setMidLineWidth(0)
        self.setObjectName("MenuPopup")
        self.setWindowFlags(QtCore.Qt.Popup | QtCore.Qt.FramelessWindowHint)
        self.setMouseTracking(True)
        self.setAcceptDrops(True)
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.setWindowOpacity(0.9)
        self.setFixedWidth(150)
        self._layout = QtWidgets.QVBoxLayout()
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setLayout(self._layout)

        self.create_menu(menu_items)

    def create_menu(self, menu_items):
        for item in menu_items:
            self._layout.addWidget(item)



# SPDX-License-Identifier: MIT

import os
import numpy as np
import pytest
from pytest import approx

from xypattern import Pattern
from xypattern.pattern import BkgNotInRangeError
from xypattern.auto_background import SmoothBrucknerBackground

from ...model.util.PeakShapes import gaussian

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, "../data")


def test_loading_chi_file():
    spec = Pattern()
    x, y = spec.data

    spec.load(os.path.join(data_path, "pattern_001.chi"))
    new_x, new_y = spec.data

    assert len(x) != len(new_x)
    assert len(y) != len(new_y)


def test_loading_invalid_file():
    pattern = Pattern()
    with pytest.raises(ValueError):
        pattern.load(os.path.join(data_path, "wrong_file_format.txt"))


def test_saving_a_file(tmp_path):
    x = np.linspace(-5, 5, 100)
    y = x**2
    pattern = Pattern(x, y)
    filename = os.path.join(tmp_path, "test.dat")
    pattern.save(filename)

    pattern2 = Pattern()
    pattern2.load(filename)

    pattern2_x, pattern2_y = pattern2.data
    assert pattern2_x == pytest.approx(x)
    assert pattern2_y == pytest.approx(y)


def test_plus_and_minus_operators():
    x = np.linspace(0, 10, 100)
    pattern1 = Pattern(x, np.sin(x))
    pattern2 = Pattern(x, np.sin(x))

    pattern3 = pattern1 + pattern2
    assert np.array_equal(pattern3.y, np.sin(x) * 2)
    assert np.array_equal(pattern2._original_y, np.sin(x) * 1)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)

    pattern3 = pattern1 + pattern1
    assert np.array_equal(pattern3.y, np.sin(x) * 2)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)

    pattern3 = pattern2 - pattern1
    assert np.array_equal(pattern3.y, np.sin(x) * 0)
    assert np.array_equal(pattern2._original_y, np.sin(x) * 1)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)

    pattern3 = pattern1 - pattern1
    assert np.array_equal(pattern3.y, np.sin(x) * 0)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)
    assert np.array_equal(pattern1._original_y, np.sin(x) * 1)


def test_plus_and_minus_operators_with_different_shapes():
    x = np.linspace(0, 10, 1000)
    x2 = np.linspace(0, 12, 1300)
    pattern1 = Pattern(x, np.sin(x))
    pattern2 = Pattern(x2, np.sin(x2))

    pattern3 = pattern1 + pattern2
    assert pattern3.x == approx(pattern1._original_x)
    assert pattern3.y == approx(pattern1._original_y * 2, abs=1e-4)

    pattern3 = pattern1 + pattern1
    assert pattern3.y == approx(np.sin(x) * 2, abs=1e-4)

    pattern3 = pattern1 - pattern2
    assert pattern3.y == approx(np.sin(x) * 0, abs=1e-4)

    pattern3 = pattern1 - pattern1
    assert pattern3.y == approx(np.sin(x) * 0, abs=1e-4)


def test_multiply_with_scalar_operator():
    x = np.linspace(0, 10, 100)
    pattern = 2 * Pattern(x, np.sin(x))
    assert np.array_equal(pattern.y, np.sin(x) * 2)


def test_using_background_pattern():
    x = np.linspace(-5, 5, 100)
    pattern_y = x**2
    bkg_y = x

    spec = Pattern(x, pattern_y)
    background_pattern = Pattern(x, bkg_y)

    spec.background_pattern = background_pattern
    new_x, new_y = spec.data

    assert np.array_equal(new_x, x)
    assert np.array_equal(new_y, pattern_y - bkg_y)


def test_using_background_pattern_with_different_spacing():
    x = np.linspace(-5, 5, 100)
    pattern_y = x**2
    x_bkg = np.linspace(-5, 5, 99)
    bkg_y = x_bkg

    spec = Pattern(x, pattern_y)
    background_pattern = Pattern(x_bkg, bkg_y)

    spec.background_pattern = background_pattern
    new_x, new_y = spec.data

    assert np.array_equal(new_x, x)
    assert np.array_equal(new_y, pattern_y - x)


def test_background_out_of_range_throws_error():
    x1 = np.linspace(0, 10)
    x2 = np.linspace(-10, -1)

    spec = Pattern(x1, x1)
    background_pattern = Pattern(x2, x2)

    with pytest.raises(BkgNotInRangeError):
        spec.background_pattern = background_pattern


def test_automatic_background_subtraction():
    x = np.linspace(0, 24, 2500)
    y = np.zeros(x.shape)

    peaks = [
        [10, 3, 0.1],
        [12, 4, 0.1],
        [12, 6, 0.1],
    ]
    for peak in peaks:
        y += gaussian(x, peak[0], peak[1], peak[2])
    y_bkg = x * 0.4 + 5.0
    y_measurement = y + y_bkg

    pattern = Pattern(x, y_measurement)

    pattern.auto_bkg = SmoothBrucknerBackground(
        smooth_width=2, iterations=50, cheb_order=50
    )

    _, y_pattern = pattern.data

    assert y_pattern == approx(y, abs=1e-4)


def test_automatic_background_subtraction_with_roi():
    x = np.linspace(0, 24, 2500)
    y = np.zeros(x.shape)

    peaks = [
        [10, 3, 0.1],
        [12, 4, 0.1],
        [12, 6, 0.1],
    ]
    for peak in peaks:
        y += gaussian(x, peak[0], peak[1], peak[2])
    y_bkg = x * 0.4 + 5.0
    y_measurement = y + y_bkg

    pattern = Pattern(x, y_measurement)

    roi = [1, 23]
    pattern.auto_bkg_roi = roi
    pattern.auto_bkg = SmoothBrucknerBackground(
        smooth_width=2, iterations=50, cheb_order=50
    )

    x_pattern, _ = pattern.data

    assert x_pattern[0] > roi[0]
    assert x_pattern[-1] < roi[1]


def test_setting_new_data():
    spec = Pattern()
    x = np.linspace(0, 10)
    y = np.sin(x)
    spec.data = x, y

    new_x, new_y = spec.data
    assert np.array_equal(new_x, x)
    assert np.array_equal(new_y, y)


def test_using_len():
    x = np.linspace(0, 10, 234)
    y = x**2
    spec = Pattern(x, y)

    assert len(spec) == 234



# SPDX-License-Identifier: MIT

import os
import pytest
from mock import MagicMock
import numpy as np

from qtpy import QtWidgets
from xypattern import Pattern

from ...controller.integration import BatchController


def test_save_xy_without_background_subtraction(
    batch_controller: BatchController, tmp_path
):
    batch_controller.model.batch_model.data = np.ones((22, 1000))
    batch_controller.model.batch_model.binning = np.arange(1000)
    QtWidgets.QFileDialog.getSaveFileName = MagicMock(
        return_value=os.path.join(tmp_path, "test.xy")
    )
    batch_controller.save_data()

    for i in range(len(batch_controller.model.batch_model.data)):
        assert os.path.exists(os.path.join(tmp_path, f"test_{i:03d}.xy"))


def test_save_xy_with_background_subtraction(
    batch_controller: BatchController, tmp_path
):
    batch_controller.model.batch_model.data = np.ones((22, 1001))
    batch_controller.model.batch_model.binning = np.linspace(0, 10, 1001)
    batch_controller.model.pattern_model.set_auto_background_subtraction(
        parameters=[2, 50, 50], roi=[2, 8]
    )

    assert batch_controller.model.pattern_model.pattern.auto_bkg

    QtWidgets.QFileDialog.getSaveFileName = MagicMock(
        return_value=os.path.join(tmp_path, "test.xy")
    )
    batch_controller.save_data()

    # check that normal patterns are saved
    for i in range(len(batch_controller.model.batch_model.data)):
        assert os.path.exists(os.path.join(tmp_path, f"test_{i:03d}.xy"))

    bkg_subtracted_path = os.path.join(tmp_path, "bkg_subtracted")
    assert os.path.exists(bkg_subtracted_path)

    # check that background patterns are saved
    for i in range(len(batch_controller.model.batch_model.data)):
        assert os.path.exists(os.path.join(bkg_subtracted_path, f"test_{i:03d}.xy"))

    # check that background subtracted patterns are saved with the reduced range
    bkg_subtracted_pattern = Pattern()
    bkg_subtracted_pattern.load(os.path.join(bkg_subtracted_path, f"test_011.xy"))
    assert len(bkg_subtracted_pattern.x) == 599
    assert len(bkg_subtracted_pattern.y) == 599
    assert np.sum(bkg_subtracted_pattern.y) == pytest.approx(0, abs=1e-10)

    # check that originalfpytterns are saved with the full range
    pattern = Pattern()
    pattern.load(os.path.join(tmp_path, f"test_011.xy"))
    assert len(pattern.x) == 1001
    assert len(pattern.y) == 1001



# SPDX-License-Identifier: MIT

import pytest
from pytest import approx
import os
import numpy as np
from xypattern.auto_background import SmoothBrucknerBackground

from ...model.PatternModel import PatternModel
from ...model.util.PeakShapes import gaussian

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, "../data")


@pytest.fixture
def pattern_model():
    return PatternModel()


def test_set_pattern(pattern_model: PatternModel):
    x = np.linspace(0.1, 15, 100)
    y = np.sin(x)
    pattern_model.set_pattern(x, y, "hoho")
    assert pattern_model.get_pattern().x == approx(x)
    assert pattern_model.get_pattern().y == approx(y)
    assert pattern_model.get_pattern().name == "hoho"


def test_load_pattern(pattern_model: PatternModel):
    pattern_model.load_pattern(os.path.join(data_path, "pattern_001.xy"))
    assert pattern_model.get_pattern().name == "pattern_001"
    assert len(pattern_model.get_pattern().x) > 101
    assert len(pattern_model.get_pattern().y) > 101


def test_auto_background_subtraction(pattern_model: PatternModel):
    x = np.linspace(0, 24, 2500)
    y = np.zeros(x.shape)

    peaks = [
        [10, 3, 0.1],
        [12, 4, 0.1],
        [12, 6, 0.1],
    ]
    for peak in peaks:
        y += gaussian(x, peak[0], peak[1], peak[2])
    y_bkg = x * 0.4 + 5.0
    y_measurement = y + y_bkg

    pattern_model.set_pattern(x, y_measurement)

    auto_background_subtraction_parameters = [2, 50, 50]
    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters
    )

    x_spec, y_spec = pattern_model.pattern.data

    assert np.sum(y_spec - y) == approx(0, abs=1e-9)


def test_auto_background_subtraction_with_out_of_range_roi(pattern_model: PatternModel):
    x = np.linspace(0, 24, 2500)
    y = np.zeros(x.shape)
    x_step = x[1] - x[0]
    x_end = x[-1]

    pattern_model.set_pattern(x, y)

    auto_background_subtraction_parameters = [2, 50, 50]
    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters, roi=[25, 26]
    )

    assert pattern_model.pattern.auto_bkg_roi == [
        x_end - 1.5 * x_step,
        x_end + x_step / 2,
    ]

    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters, roi=[50, 30]
    )
    assert pattern_model.pattern.auto_bkg_roi == [
        x_end - 1.5 * x_step,
        x_end + x_step / 2,
    ]

    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters, roi=[23, 30]
    )
    assert pattern_model.pattern.auto_bkg_roi == [23, x_end + x_step / 2]

    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters, roi=[-10, 23]
    )
    assert pattern_model.pattern.auto_bkg_roi == [x[0] - x_step / 2, 23]

    pattern_model.set_auto_background_subtraction(
        auto_background_subtraction_parameters, roi=[-10, -3]
    )
    assert pattern_model.pattern.auto_bkg_roi == [
        x[0] - x_step / 2,
        x[0] + 1.5 * x_step,
    ]



# SPDX-License-Identifier: MIT
import os
import numpy as np

from pytest import approx

try:
    from urllib import pathname2url
except ImportError:
    from urllib.request import pathname2url

from CifFile import ReadCif

from ...model.util.cif import CifPhase, CifConverter

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')
cif_path = os.path.join(data_path, 'cif')


def get_cif_url(cif_filename):
    file_path = 'file:' + pathname2url(
        os.path.join(cif_path, cif_filename))
    return file_path


def test_reading_cif():
    fcc_cif = ReadCif(get_cif_url('fcc.cif'))

    cif_phase = CifPhase(fcc_cif[fcc_cif.keys()[0]])

    assert cif_phase.a == 4.874
    assert cif_phase.b == 4.874
    assert cif_phase.c == 4.874

    assert cif_phase.alpha == 90
    assert cif_phase.beta == 90
    assert cif_phase.gamma == 90

    assert np.round(cif_phase.volume, 2) == 115.79
    assert cif_phase.space_group_number == 225

    assert len(cif_phase.atoms) == 8
    assert cif_phase.comments == 'HoN, Fm-3m - NaCl structure type, ICSD 44776'


def test_read_cif_without_cell_volume():
    fcc_cif = ReadCif(get_cif_url('ICSD_triclinic_without_cell_volume.cif'))
    cif_phase = CifPhase(fcc_cif[fcc_cif.keys()[0]])
    assert cif_phase.volume == approx(465.74, 1e-2)


def test_calculating_xrd_pattern_from_cif_file():
    fcc_cif = ReadCif(get_cif_url('fcc.cif'))
    cif_phase = CifPhase(fcc_cif[fcc_cif.keys()[0]])
    cif_converter = CifConverter(0.31)
    jcpds_phase = cif_converter.convert_cif_phase_to_jcpds(cif_phase)

    assert jcpds_phase.reflections[0].intensity == 100
    assert jcpds_phase.reflections[0].d0 == approx(2.814, 1e-5)
    assert jcpds_phase.reflections[1].d0 == approx(2.437, 1e-5)


def test_loading_cif_phase_and_calculate_jcpds():
    cif_converter = CifConverter(0.31)
    jcpds_phase = cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'fcc.cif'))
    assert jcpds_phase.name == 'fcc'
    assert jcpds_phase.reflections[0].intensity == 100
    assert jcpds_phase.reflections[0].d0 == approx(2.814, 1e-5)
    assert jcpds_phase.reflections[1].d0 == approx(2.437, 1e-5)


def test_loading_cif_phase_with_occupancies_specified():
    cif_converter = CifConverter(0.31)
    jcpds_phase = cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'magnesiowustite.cif'))
    assert jcpds_phase.reflections[0].intensity == approx(27.53, 0.2)
    assert jcpds_phase.reflections[1].intensity == 100
    assert jcpds_phase.reflections[2].intensity == approx(65.02, 0.2)


def test_reading_american_mineralogist_db_cif():
    cif_converter = CifConverter(0.31)
    jcpds_phase = cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'amcsd.cif'))
    assert jcpds_phase.params['a0'] == 5.4631
    assert jcpds_phase.reflections[0].intensity == approx(73.48, 0.5)
    assert jcpds_phase.reflections[1].intensity == 100
    assert jcpds_phase.reflections[2].intensity == approx(33.6, 0.5)
    assert jcpds_phase.reflections[3].intensity == approx(14.65, 0.5)


def test_read_cif_with_errors_in_atomic_positions():
    cif_converter = CifConverter(0.31, min_d_spacing=1.5, min_intensity=10)
    jcpds_phase = cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'apatite.cif'))
    assert jcpds_phase.params['a0'] == 9.628


def test_read_cif_from_shelx():
    cif_converter = CifConverter(0.31)
    jcpds_phase = cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'Fe2O3_shelx.cif'))
    assert jcpds_phase.params['a0'] == 6.524
    assert jcpds_phase.params['b0'] == 4.702
    assert jcpds_phase.params['c0'] == 4.603


def test_convert_cif_with_triclinic_geometry():
    cif_converter = CifConverter(0.31, min_d_spacing=1, min_intensity=5)
    cif_converter.convert_cif_to_jcpds(os.path.join(cif_path, 'ICSD_triclinic.cif'))



# SPDX-License-Identifier: MIT

import os
import numpy as np
import pytest
from mock import MagicMock

from qtpy import QtWidgets

from ..utility import unittest_data_path, click_button


def test_configuration_selected_changes_background_image_widgets(
    background_controller, integration_widget, dioptas_model, qtbot
):
    widget = integration_widget
    model = dioptas_model

    model.img_model.load(os.path.join(unittest_data_path, "image_001.tif"))
    model.img_model.load_background(os.path.join(unittest_data_path, "image_001.tif"))

    model.add_configuration()
    model.img_model.load(os.path.join(unittest_data_path, "image_001.tif"))
    model.img_model.load_background(os.path.join(unittest_data_path, "image_002.tif"))

    assert str(widget.bkg_image_filename_lbl.text()) == "image_002.tif"

    model.select_configuration(0)
    assert str(widget.bkg_image_filename_lbl.text()) == "image_001.tif"

    widget.bkg_image_offset_sb.setValue(100)
    model.select_configuration(1)
    assert widget.bkg_image_offset_sb.value() == 0

    widget.bkg_image_scale_sb.setValue(2)
    model.select_configuration(0)
    assert widget.bkg_image_scale_sb.value() == 1


def test_configuration_selected_changes_auto_background_widgets(
    background_controller, pattern_controller, integration_widget, dioptas_model, qtbot
):
    widget = integration_widget
    model = dioptas_model

    model.pattern_model.load_pattern(
        os.path.join(unittest_data_path, "pattern_001.chi")
    )
    click_button(widget.qa_bkg_pattern_btn)
    assert widget.bkg_pattern_gb.isChecked()

    model.add_configuration()
    assert model.pattern_model.pattern.auto_bkg is None
    assert not widget.bkg_pattern_gb.isChecked()

    model.select_configuration(0)
    assert widget.bkg_pattern_gb.isChecked()
    click_button(widget.qa_bkg_pattern_inspect_btn)

    model.select_configuration(1)
    assert not widget.bkg_pattern_gb.isChecked()


def test_changing_unit(
    background_controller, pattern_controller, integration_widget, dioptas_model, qtbot
):
    widget = integration_widget
    model = dioptas_model

    model.calibration_model.load(
        os.path.join(unittest_data_path, "LaB6_40keV_MarCCD.poni")
    )
    model.img_model.load(os.path.join(unittest_data_path, "image_001.tif"))

    x_raw, y_raw = model.pattern_model.pattern.data
    click_button(widget.qa_bkg_pattern_btn)
    x_2th, y_2th = model.pattern_model.pattern.data

    assert np.sum(y_raw) != np.sum(y_2th)

    click_button(widget.pattern_q_btn)
    x_q, y_q = model.pattern_model.pattern.data
    x_q_bkg, y_q_bkg = model.pattern_model.pattern.auto_background_pattern.data

    assert np.max(x_q) < np.max(x_2th)
    assert x_q[0] == x_q_bkg[0]
    assert x_q[-1] == x_q_bkg[-1]


def test_save_fit_background_pattern(
    background_controller,
    pattern_controller,
    integration_widget,
    dioptas_model,
    qtbot,
    tmpdir,
):
    widget = integration_widget
    model = dioptas_model

    QtWidgets.QFileDialog.getSaveFileName = MagicMock(
        return_value=os.path.join(tmpdir, "test_bg.xy")
    )
    model.calibration_model.create_file_header = MagicMock(return_value="None")
    click_button(widget.qa_bkg_pattern_btn)
    click_button(widget.bkg_pattern_save_btn)
    assert os.path.exists(os.path.join(tmpdir, "test_bg.xy"))



# SPDX-License-Identifier: MIT

import os

from ..utility import QtTest, click_button, delete_if_exists
from mock import MagicMock

from qtpy import QtWidgets

from ...model.util.calc import convert_units

from ...widgets.integration import IntegrationWidget
from ...controller.integration.PatternController import PatternController
from ...model.DioptasModel import DioptasModel

unittest_data_path = os.path.join(os.path.dirname(__file__), "../data")


class PatternControllerTest(QtTest):
    def setUp(self):
        self.widget = IntegrationWidget()
        self.model = DioptasModel()

        self.controller = PatternController(
            widget=self.widget, dioptas_model=self.model
        )

    def tearDown(self):
        delete_if_exists(os.path.join(unittest_data_path, "test.xy"))

    def test_configuration_selected_changes_active_unit_btn(self):
        self.model.calibration_model.load(
            os.path.join(unittest_data_path, "CeO2_Pilatus1M.poni")
        )
        self.model.img_model.load(
            os.path.join(unittest_data_path, "CeO2_Pilatus1M.tif")
        )

        self.model.add_configuration()
        click_button(self.widget.pattern_q_btn)

        self.model.add_configuration()
        click_button(self.widget.pattern_d_btn)

        self.model.select_configuration(0)
        self.assertTrue(self.widget.pattern_tth_btn.isChecked())
        self.assertFalse(self.widget.pattern_q_btn.isChecked())
        self.assertFalse(self.widget.pattern_d_btn.isChecked())

        self.assertEqual(
            self.widget.pattern_widget.pattern_plot.getAxis("bottom").labelString(),
            "<span style='color: #ffffff'>2θ (°)</span>",
        )

        self.model.select_configuration(1)
        self.assertTrue(self.widget.pattern_q_btn.isChecked())

        self.assertEqual(
            self.widget.pattern_widget.pattern_plot.getAxis("bottom").labelString(),
            "<span style='color: #ffffff'>Q (Å⁻¹)</span>",
        )

        self.model.select_configuration(2)
        self.assertTrue(self.widget.pattern_d_btn.isChecked())
        self.assertEqual(
            self.widget.pattern_widget.pattern_plot.getAxis("bottom").labelString(),
            "<span style='color: #ffffff'>d (Å)</span>",
        )

    def test_save_pattern_without_background(self):
        QtWidgets.QFileDialog.getSaveFileName = MagicMock(
            return_value=os.path.join(unittest_data_path, "test.xy")
        )
        self.model.calibration_model.create_file_header = MagicMock(return_value="None")
        click_button(self.widget.qa_save_pattern_btn)

        self.assertTrue(os.path.exists(os.path.join(unittest_data_path, "test.xy")))

    def test_click_pattern_sends_tth_changed_signal(self):
        self.model.clicked_tth_changed.emit = MagicMock()
        self.widget.integration_pattern_widget.pattern_view.mouse_left_clicked.emit(
            10, 300
        )
        self.model.clicked_tth_changed.emit.assert_called_once_with(10)

    def test_click_pattern_changes_position_line(self):
        self.widget.integration_pattern_widget.pattern_view.mouse_left_clicked.emit(
            10, 300
        )
        prior_pos = (
            self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()
        )
        self.widget.integration_pattern_widget.pattern_view.mouse_left_clicked.emit(
            12, 300
        )
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertEqual(new_pos, 12)

    def test_pos_line_updates_on_integration_unit_change(self):
        self.load_pilatus1M_image_and_calibration()
        self.widget.integration_pattern_widget.pattern_view.mouse_left_clicked.emit(
            10, 300
        )

        click_button(self.widget.integration_pattern_widget.q_btn)
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertEqual(
            new_pos,
            convert_units(
                10, self.model.calibration_model.wavelength, "2th_deg", "q_A^-1"
            ),
        )

        click_button(self.widget.integration_pattern_widget.d_btn)
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertAlmostEqual(
            new_pos,
            convert_units(
                10, self.model.calibration_model.wavelength, "2th_deg", "d_A"
            ),
        )

        click_button(self.widget.integration_pattern_widget.tth_btn)
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertAlmostEqual(new_pos, 10)

    def test_pos_line_updates_on_tth_clicked_signal(self):
        self.load_pilatus1M_image_and_calibration()
        self.model.clicked_tth_changed.emit(5)
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertEqual(new_pos, 5)

        click_button(self.widget.integration_pattern_widget.q_btn)
        self.model.clicked_tth_changed.emit(3)
        new_pos = self.widget.integration_pattern_widget.pattern_view.pos_line.getPos()[
            0
        ]
        self.assertAlmostEqual(
            new_pos,
            convert_units(
                3, self.model.calibration_model.wavelength, "2th_deg", "q_A^-1"
            ),
        )

    def load_pilatus1M_image_and_calibration(self):
        file_name = os.path.join(unittest_data_path, "CeO2_Pilatus1M.tif")
        self.model.img_model.load(file_name)
        calibration_file_name = os.path.join(unittest_data_path, "CeO2_Pilatus1M.poni")
        self.model.calibration_model.load(calibration_file_name)



# SPDX-License-Identifier: MIT
"""Test AutoLevel computation"""
import numpy as np
import pytest

from dioptas.widgets.plot_widgets.utils import AutoLevel


def test_default_mode():
    auto_level = AutoLevel()
    data = np.arange(1001).reshape(1, -1)
    range_ = auto_level.get_range(data)
    # Default uses 0.4th / 99.6th percentile
    assert range_ == (4.0, 996.0)


@pytest.mark.parametrize(
    "data",
    [np.array([[1, 2, 3]]), np.array([[np.inf, 3, 1, np.nan, -np.inf]])],
)
def test_minmax_mode(data):
    auto_level = AutoLevel()
    auto_level.mode = "minmax"
    data = np.array(data)
    range_ = auto_level.get_range(data)
    assert range_ == (1, 3)


def test_mean3std_mode():
    auto_level = AutoLevel()
    auto_level.filter_dummy = False
    auto_level.mode = "mean3std"
    data = np.ones(1000) * 100
    data[0:2] = 1, 1000
    data.shape = 1, -1
    range_ = auto_level.get_range(data)
    mean, std = np.mean(data), np.std(data)
    assert range_ == (mean - 3 * std, mean + 3 * std)


def test_mean3std_mode_clipped_to_minmax():
    auto_level = AutoLevel()
    auto_level.mode = "mean3std"
    data = np.array([[1, 2, 3, np.inf, np.nan, -np.inf]])
    range_ = auto_level.get_range(data)
    assert range_ == (1, 3)


@pytest.mark.parametrize(
    "mode",
    ["1percentile", "1.percentile", "1.000percentile"],
)
def test_percentil_mode(mode):
    auto_level = AutoLevel()
    auto_level.mode = mode
    data = np.arange(101).reshape(1, -1)
    range_ = auto_level.get_range(data)
    assert range_ == (1, 99)


def test_none_data():
    auto_level = AutoLevel()
    range_ = auto_level.get_range(None)
    assert range_ is None


def test_no_finite_data():
    auto_level = AutoLevel()
    data = np.array([[np.inf, np.nan, -np.inf]])
    range_ = auto_level.get_range(data)
    assert range_ is None


def test_unsupported_autoscale_mode():
    auto_level = AutoLevel()
    auto_level.mode = "unsupported"
    data = np.array([[1, 2, 3]])
    with pytest.raises(ValueError):
        range_ = auto_level.get_range(data)


def test_filter_dummy():
    auto_level = AutoLevel()
    auto_level.mode = "minmax"
    auto_level.filter_dummy = True
    data = np.array([
        [ 0,  1,  2,  3,  4],
        [-1, -1, -1, -1, -1],
        [-1, -1, -1, -1, -1],
        [ 4,  3,  2,  1,  0],
    ])
    range_ = auto_level.get_range(data)
    assert range_ == (0, 4)

    range_ = auto_level.get_range(np.transpose(data))
    assert range_ == (0, 4)



# SPDX-License-Identifier: MIT

try:
    from extra_data import H5File
    from extra_data.exceptions import FileStructureError
    extra_data_installed = True
except ImportError:
    extra_data_installed = False

__all__ = ['KaraboFile', 'extra_data_installed']


class KaraboFile:
    def __init__(self, filename, source_ind=0):
        if not extra_data_installed:
            raise IOError('extra_data is required to load karabo h5 files')
        try:
            self.f = H5File(filename)
        except FileStructureError:
            raise IOError('This hdf5 file is not a generated by karabo')
        self.series_max = len(self.f.train_ids)
        self.sources = [s for s in self.f.instrument_sources if "daqOutput" in s]
        self.current_source = self.sources[source_ind]

    def get_image(self, ind):
        sel = self.f.select(self.current_source, 'data.image.pixels')
        tid, data = sel.train_from_index(ind)
        return data[self.current_source]['data.image.pixels'][::-1]



# SPDX-License-Identifier: MIT

import os
import shutil
import unittest

from ...model.util.NewFileWatcher import NewFileInDirectoryWatcher

unittest_data_path = os.path.join(os.path.dirname(__file__), '../data')


@unittest.skip('inotify Limit does not allow to run this on CI')
def test_getting_callback_for_new_file(tmp_path):
    directory_watcher = NewFileInDirectoryWatcher()

    def callback_fcn(filepath):
        assert filepath == os.path.abspath(os.path.join(tmp_path, 'image_003.tif'))

    directory_watcher.path = tmp_path
    directory_watcher.file_added.connect(callback_fcn)
    directory_watcher.file_types.add('.tif')
    directory_watcher.activate()

    shutil.copy2(os.path.join(unittest_data_path, 'image_001.tif'),
                 os.path.join(tmp_path, 'image_003.tif'))

    directory_watcher.deactivate()


@unittest.skip('inotify Limit does not allow to run this on CI')
def test_filename_is_emitted_with_full_file_available(tmp_path):
    directory_watcher = NewFileInDirectoryWatcher()
    original_path = os.path.join(unittest_data_path, 'image_001.tif')
    destination_path = os.path.join(tmp_path, 'image_003.tif')
    original_filesize = os.stat(original_path).st_size

    def callback_fcn(filepath):
        filesize = os.stat(filepath).st_size
        assert filesize == original_filesize

    directory_watcher.path = tmp_path
    directory_watcher.file_added.connect(callback_fcn)
    directory_watcher.file_types.add('.tif')
    directory_watcher.activate()

    shutil.copy2(original_path, destination_path)

    directory_watcher.deactivate()



# SPDX-License-Identifier: MIT

from qtpy import QtWidgets, QtCore, QtGui

from .ImageWidget import ImageWidget
from .PatternWidget import PatternWidget
from .OverlayWidget import OverlayWidget
from .PhaseWidget import PhaseWidget
from .CorrectionsWidget import CorrectionsWidget
from .BackgroundWidget import BackgroundWidget
from .OptionsWidget import OptionsWidget


class IntegrationControlWidget(QtWidgets.QWidget):
    def __init__(self):
        super(IntegrationControlWidget, self).__init__()

        self._layout = QtWidgets.QHBoxLayout()
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(5)

        self.tab_widget_1 = TabWidgetMinSize()
        self.tab_widget_2 = TabWidgetMinSize()

        self.img_control_widget = ImageWidget()
        self.pattern_control_widget = PatternWidget()
        self.overlay_control_widget = OverlayWidget()
        self.phase_control_widget = PhaseWidget()
        self.corrections_control_widget = CorrectionsWidget()
        self.background_control_widget = BackgroundWidget()
        self.integration_options_widget = OptionsWidget()

        self.horizontal_splitter = QtWidgets.QSplitter()
        self.horizontal_splitter.setOrientation(QtCore.Qt.Horizontal)

        self.horizontal_splitter.addWidget(self.tab_widget_1)
        self.horizontal_splitter.addWidget(self.tab_widget_2)
        self.horizontal_splitter.setCollapsible(0, False)
        self.horizontal_splitter.setCollapsible(1, False)

        self.vertical_splitter = QtWidgets.QSplitter()
        self.vertical_splitter.setOrientation(QtCore.Qt.Vertical)

        self.vertical_splitter.addWidget(self.horizontal_splitter)

        self._layout.addWidget(self.vertical_splitter)
        self.setLayout(self._layout)

        self.current_layout = None

        self.orientation = QtCore.Qt.Horizontal  # other value is QtCore.Qt.Horizontal

        self.tab_widget_1.addTab(self.img_control_widget, "Image")
        self.tab_widget_1.addTab(self.pattern_control_widget, "Pattern")
        self.tab_widget_1.addTab(self.overlay_control_widget, "Overlay")
        self.tab_widget_1.addTab(self.phase_control_widget, "Phase")
        self.tab_widget_1.addTab(self.corrections_control_widget, "Cor")
        self.tab_widget_1.addTab(self.background_control_widget, "Bkg")
        self.tab_widget_1.addTab(self.integration_options_widget, "X")

        self.tab_widget_1.setCurrentIndex(0)

    def update_tab_widget_1_sizes(self):
        for i in range(self.tab_widget_1.count()):
            self.tab_widget_1.widget(i).setSizePolicy(
                QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored
            )
        self.tab_widget_1.currentWidget().setSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        self.tab_widget_1.currentWidget().resize(
            self.tab_widget_1.currentWidget().minimumSizeHint()
        )
        self.tab_widget_1.currentWidget().adjustSize()

    def horizontal_layout_1(self):
        self.current_layout = 1

        self.tab_widget_2.hide()

        self.tab_widget_1.insertTab(2, self.overlay_control_widget, "Overlay")
        self.tab_widget_1.insertTab(3, self.phase_control_widget, "Phase")

        self.overlay_control_widget.overlay_header_btn.hide()
        self.phase_control_widget.phase_header_btn.hide()

    def horizontal_layout_2(self):
        self.current_layout = 2

        self.tab_widget_2.show()

        self.tab_widget_2.addTab(self.overlay_control_widget, "Overlay")
        self.tab_widget_2.addTab(self.phase_control_widget, "Phase")
        widget_width = int(self.width() / 2)
        self.horizontal_splitter.setSizes([widget_width, widget_width])

        self.overlay_control_widget.overlay_header_btn.hide()
        self.phase_control_widget.phase_header_btn.hide()

    def horizontal_layout_3(self):
        self.current_layout = 3
        self.tab_widget_2.hide()

        self.horizontal_splitter.addWidget(self.overlay_control_widget)
        self.horizontal_splitter.addWidget(self.phase_control_widget)

        self.horizontal_splitter.setCollapsible(2, False)
        self.horizontal_splitter.setCollapsible(3, False)
        widget_width = int(self.width() / 3)
        self.horizontal_splitter.setSizes([widget_width, 0, widget_width, widget_width])

        self.overlay_control_widget.show()
        self.phase_control_widget.show()

        self.overlay_control_widget.overlay_header_btn.show()
        self.phase_control_widget.phase_header_btn.show()

    def vertical_layout(self):
        self.tab_widget_2.hide()
        self.vertical_splitter.addWidget(self.overlay_control_widget)
        self.vertical_splitter.addWidget(self.phase_control_widget)

        self.overlay_control_widget.show()
        self.phase_control_widget.show()

        self.overlay_control_widget.overlay_header_btn.show()
        self.phase_control_widget.phase_header_btn.show()

    def update_layout(self, force_layout=False):
        if self.orientation == QtCore.Qt.Horizontal:
            if self.width() < 900:
                if self.current_layout != 1 or force_layout:
                    self.horizontal_layout_1()
            elif self.width() > 1400:
                if self.current_layout != 3 or force_layout:
                    self.horizontal_layout_3()
            else:
                if self.current_layout != 2 or force_layout:
                    self.horizontal_layout_2()
        elif self.orientation == QtCore.Qt.Vertical:
            self.vertical_layout()

        self.overlay_control_widget.update_overlay_tw_column_sizes()
        self.phase_control_widget.update_phase_tw_column_sizes()

    def resizeEvent(self, a0: QtGui.QResizeEvent):
        self.update_layout()
        super(IntegrationControlWidget, self).resizeEvent(a0)

    def setOrientation(self, a0):
        """
        Sets the orientation of the control widgets
        :param a0: either QtCore.Qt.Horizontal or QtCore.Qt.Vertical
        """
        self.orientation = a0
        self.update_layout(True)


class TabWidgetMinSize(QtWidgets.QTabWidget):
    def __init__(self):
        super(TabWidgetMinSize, self).__init__()
        self.currentChanged.connect(self.update_sizes)

    def update_sizes(self):
        for i in range(self.count()):
            self.widget(i).setSizePolicy(
                QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored
            )

        if self.currentWidget() is not None:
            self.currentWidget().setSizePolicy(
                QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
            )
            self.currentWidget().resize(self.currentWidget().minimumSizeHint())
            self.currentWidget().adjustSize()

    def setCurrentIndex(self, p_int):
        self.update_sizes()
        super(TabWidgetMinSize, self).setCurrentIndex(p_int)



import os
from dioptas.model.util.cosmics import cosmicsimage
from dioptas.model.ImgModel import ImgModel

from ..utility import unittest_data_path


def test_cosmics_image():
    img_model = ImgModel()
    img_model.load(os.path.join(unittest_data_path, 'image_001.tif'))
    test = cosmicsimage(img_model.img_data, sigclip=3.0, objlim=3.0)
    test.lacosmiciteration(True)
    test.clean()
    assert test is not None
    assert test.mask.shape == img_model.img_data.shape



# SPDX-License-Identifier: MIT

from ..utility import QtTest, click_button
import os
import gc
import unittest

from qtpy import QtWidgets
from mock import MagicMock, patch

from ..utility import enter_value_into_text_field

from ...controller.integration import BackgroundController
from ...controller.integration import PatternController
from ...model.DioptasModel import DioptasModel
from ...widgets.integration import IntegrationWidget

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')


class IntegrationBackgroundControllerTest(QtTest):

    def setUp(self):
        # Mocking the function which will block the unittest for some reason...
        # Use patch to ensure proper cleanup
        self.processEvents_patcher = patch.object(QtWidgets.QApplication, 'processEvents', MagicMock())
        self.processEvents_patcher.start()

        self.widget = IntegrationWidget()
        self.model = DioptasModel()

        self.pattern_controller = PatternController(self.widget, self.model)
        self.background_controller = BackgroundController(self.widget, self.model)
        self.overlay_tw = self.widget.overlay_tw

    def tearDown(self):
        # Stop the patcher to restore original behavior
        self.processEvents_patcher.stop()

        # Clean up widgets
        self.widget.close()
        self.widget.deleteLater()
        del self.pattern_controller
        del self.background_controller
        del self.widget
        del self.model
        gc.collect()

    def test_pattern_bkg_toggle_inspect_button(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        click_button(self.widget.qa_bkg_pattern_btn)
        click_button(self.widget.qa_bkg_pattern_inspect_btn)
        x_bkg, y_bkg = self.widget.pattern_widget.bkg_item.getData()
        self.assertGreater(len(x_bkg), 0)

        click_button(self.widget.qa_bkg_pattern_inspect_btn)
        x_bkg, y_bkg = self.widget.pattern_widget.bkg_item.getData()
        self.assertIsNone(x_bkg)
        self.assertIsNone(y_bkg)

    def test_pattern_bkg_inspect_btn_is_untoggled_when_disabling_pattern_gb(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.widget.bkg_pattern_gb.setChecked(True)
        self.widget.bkg_pattern_inspect_btn.toggle()
        self.widget.bkg_pattern_gb.setChecked(False)
        x_bkg, y_bkg = self.widget.pattern_widget.bkg_item.getData()
        self.assertIsNone(x_bkg)
        self.assertIsNone(y_bkg)

    def test_pattern_bkg_linear_region_changes_txt_fields(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.widget.bkg_pattern_gb.setChecked(True)
        self.widget.bkg_pattern_inspect_btn.toggle()

        self.widget.pattern_widget.set_bkg_roi(5, 11)

        x_min = float(str(self.widget.bkg_pattern_x_min_txt.text()))
        x_max = float(str(self.widget.bkg_pattern_x_max_txt.text()))

        self.assertAlmostEqual(x_min, 5,  delta=0.02)
        self.assertAlmostEqual(x_max, 11, delta=0.02)

    def test_pattern_bkg_txt_fields_change_linear_regions(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.widget.bkg_pattern_gb.setChecked(True)
        self.widget.bkg_pattern_inspect_btn.toggle()

        enter_value_into_text_field(self.widget.bkg_pattern_x_min_txt, '5')
        enter_value_into_text_field(self.widget.bkg_pattern_x_max_txt, '11')

        x_min, x_max = self.widget.pattern_widget.bkg_roi.getRegion()

        self.assertAlmostEqual(x_min, 5,  delta=0.02)
        self.assertAlmostEqual(x_max, 11, delta=0.02)

    def test_pattern_bkg_range_remains_when_loading_new_pattern(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.widget.bkg_pattern_gb.setChecked(True)
        self.widget.bkg_pattern_inspect_btn.toggle()

        enter_value_into_text_field(self.widget.bkg_pattern_x_min_txt, '5')
        enter_value_into_text_field(self.widget.bkg_pattern_x_max_txt, '11')

        x_min = self.widget.bkg_pattern_x_min_txt.text()
        x_max = self.widget.bkg_pattern_x_max_txt.text()

        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.assertEqual(x_min, self.widget.bkg_pattern_x_min_txt.text())
        self.assertEqual(x_max, self.widget.bkg_pattern_x_max_txt.text())



if __name__ == '__main__':
    unittest.main()
